;ͻ
;Metal Warrior III by Cadaver: Actor routines                                 
;Started in 6/2000!                                                           
;ͼ

AB_JUMP         = 1

MAXITEMSPR      = 21

;Ŀ
;INITACTORS                                                                   
;                                                                             
;Clears every actor.                                                          
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X                                                                
;

initactors:     ldx #MAXACT-1
initactor_loop: lda #ACT_NONE
                sta actt,x
                sta acthp,x
                lda #$ff
                sta actorg,x
                dex
                bpl initactor_loop
                rts

;Ŀ
;MOVEACTORS                                                                   
;                                                                             
;Calls move routine of each actor.                                            
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

moveactors:     ldx #$00
                stx npccollnum                  ;Reset NPC-collision
                stx bossfight

;Ŀ
;FLASHITEMCOLORS                                                              
;                                                                             
;Does what the name says :-)                                                  
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

flashitemcolors:lda itemflashcolor
                lsr
                lsr
                lsr
                and #$03
                tay
                lda itemflashcoltbl,y
N               set 0
                repeat MAXITEMSPR
                sta sprcoltbl-$40+ITEMB+N
N               set N+1
                repend
                inc itemflashcolor
                ldx #MAXACT-1
moveact_loop:   ldy actt,x                      ;Actor exists? (type nonzero)
                beq moveact_next
                lda atroutlo-1,y                ;Get move routine address
                sta moveact_jsr+1
                lda atrouthi-1,y
                sta moveact_jsr+2
moveact_jsr:    jsr $0000                       ;Perform actor move
                txa                             ;Player won't be removed
                beq moveact_next
                ldy actt,x
                cpy #ACT_LASTENEMY+1
                bcs moveact_nobosscheck
                lda atarmor-1,y                 ;Is it a boss (armored)?
                bne moveact_next                ;Then no removal
moveact_nobosscheck:
                lda actxh,x
                cmp actleftlimit
                bcc moveact_removeactor
                cmp actrightlimit
                bcs moveact_removeactor
                lda actyh,x
                cmp acttoplimit
                bcc moveact_removeactor
                cmp actbottomlimit
                bcc moveact_next
moveact_removeactor:
                jsr removeactor
moveact_next:   dex
                bpl moveact_loop
moveact_end:    rts

;Ŀ
;REMOVEALLACTORS                                                              
;                                                                             
;Removes all actors (except player), returning them to leveldata.             
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

removeallactors:ldx #MAXACT-1
raa_loop:       lda actt,x
                beq raa_skip
                jsr removeactor
raa_skip:       dex
                bne raa_loop
                rts

;Ŀ
;REMOVEACTOR                                                                  
;                                                                             
;Removes actor from screen and returns it to leveldata.                       
;                                                                             
;Parameters: X:Actor number                                                   
;Returns: -                                                                   
;Modifies: A,Y,temp1                                                          
;

removeactor:    ldy actorg,x                    ;Origin in the leveldata?
                cpy #$ff
                beq ract_noorigin
                jsr storeactorpos
                lda actt,x
                sta lvlactt,y                   ;Store type
                cmp #ACT_ITEM
                beq ract_item
                cmp #ACT_SHOPKEEPER
                beq ract_item
                lda acthp,x                     ;Store hitpoints
ract_common:    sta lvlactp,y                   ;(possibly changed)
ract_noorigin:  lda #$00
                sta actt,x
                cpx #ACTI_PLAYER                ;If player, don't clear hit-
                beq ract_skiphp                 ;points
                sta acthp,x
ract_skiphp:    rts
ract_item:      lda actf,x                      ;Item holds its type in the
                jmp ract_common                 ;frame

storeactorpos:  lda actxh,x                     ;Store coordinates
                sta lvlactx,y                   ;(possibly changed)
                lda actyh,x
                sta lvlacty,y
                lda actxl,x
                lsr
                lsr
                lsr
                lsr
                sta temp1
                lda actyl,x
                and #$f0
                ora temp1
                sta lvlactb,y
                rts

;Ŀ
;DRAWACTORS                                                                   
;                                                                             
;Draws all actors (sprites & chars), or actually prepares the information for 
;the drawroutines :-)                                                         
;Will then jump to DRAWPLAYERGUN.                                             
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y,temp1-temp7                                                  
;

drawactors:     lda #$00
                sta numcharactors
                sta numsprites
                ldx #MAXACT-1
doact_loop:     ldy actt,x                      ;Actor exists? (type nonzero)
                bne doact_notzero
                dex
                bpl doact_loop
                bmi doact_done
doact_restx:    ldx #$00
doact_next:     dex
                bpl doact_loop
doact_done:     jmp drawplayergun

doact_char:     lda actf,x                      ;Negative frame = invisible
                bmi doact_next
                clc
                adc atrightframe-1,y
                ldy numcharactors
                sta charnum,y
                lda actyl,x                     ;Take actor Y coordinate                sbc sprsubyl
                sbc charsubyl                   ;and subtract scrolling
                php
                rol
                rol
                rol
                and #$03
                plp
                sta doact_ora+1
                lda actyh,x
                sbc charsubyh
                asl
                asl
doact_ora:      ora #$00
                cmp #SCROLLROWS                 ;Check Y limit
                bcs doact_next
                ldy numcharactors
                sta charrow,y
                lda actxl,x                     ;Take actor X coordinate
                sbc charsubxl                   ;and subtract scrolling
                php
                rol
                rol
                rol
                and #$03
                plp
                sta doact_ora2+1
                lda actxh,x
                sbc charsubxh
                cmp #$0a                        ;Check X limit
                bcs doact_next
                asl
                asl
doact_ora2:     ora #$00
                ldy numcharactors
                sta charcolumn,y
                inc numcharactors
                bne doact_next

doact_notzero:  lda atformhi-1,y                ;If sprite formation addr.
                beq doact_char                  ;zero, it is a char
                sta doact_sprjump+2
                lda atformlo-1,y                ;Get sprite formation type
                sta doact_sprjump+1             ;(jump address)
                stx doact_restx+1

doact_sprite:   lda athotspotx-1,y
                sta doact_hotspotx+1
                lda athotspoty-1,y
                sta doact_hotspoty+1
                lda atframelo-1,y               ;Get frame table address
                sta temp5
                lda atframehi-1,y
                sta temp6
                lda actd,x
                asl                             ;Carry holds actor direction
                lda actf,x
                bmi doact_flash                 ;Flashing
                bcs doact_left
                adc atrightframe-1,y
                sec
doact_left:     sta temp7

                lda actxl,x                     ;Take actor coordinates
                sbc sprsubxl                    ;Perform scrolling-subtract
                sta temp1
                lda actxh,x
                sbc sprsubxh
                and #$3f
                tay
                lda temp1
                lsr
                lsr
                lsr
                ora coordtbllo,y
                sec
doact_hotspotx: sbc #$00
                sta temp1
                lda coordtblhi,y
                sbc #$00
                sta temp2

                lda actyl,x                     ;Take actor coordinates
                sec
                sbc sprsubyl                    ;Perform scrolling-subtract
                sta temp3
                lda actyh,x
                sbc sprsubyh
                and #$3f
                tay
                lda temp3
                lsr
                lsr
                lsr
                ora coordtbllo,y
                sec
doact_hotspoty: sbc #$00
                sta temp3
                lda coordtblhi,y
                sbc #$00

                ldx numsprites                  ;Get first free spritenumber
doact_sprjump:  jmp $0000                       ;Everything ready for jump
doact_flash:    jmp doact_restx

doact_1x1:      cpx #MAXSPR                     ;Enough free sprites?
                bcs doact_1x1skip
doact_1x1ok:    tay                             ;Check Y-coord high
                bne doact_1x1skip
                lda temp1                       ;Store coordinates
                sta sprxl,x
                lda temp2
                sta sprxh,x
                lda temp3
                sta spry,x
                ldy temp7                       ;Get right frameindex
                lda (temp5),y
                sta sprf,x
                inx
                stx numsprites
doact_1x1skip:  jmp doact_restx

doact_1x2:      sta temp4
                cpx #MAXSPR-1                   ;Enough free sprites?
                bcs doact_1x2totalskip
doact_1x2ok:    lda temp1                       ;Store coordinates
                sta sprxl,x
                sta sprxl+1,x
                lda temp2
                sta sprxh,x
                sta sprxh+1,x
                lda temp7
                asl                             ;Get right frameindex
                tay
                lda temp3
                sta spry,x
                adc #$15                        ;Carry is still clear :-)
                sta spry+1,x
                lda (temp5),y
                sta sprf,x
                iny
                lda (temp5),y
                sta sprf+1,x
                lda temp4
                bne doact_1x2totalskip
                inx
                adc #$00
                bne doact_1x2skip
                inx
doact_1x2skip:  stx numsprites
doact_1x2totalskip:
                jmp doact_restx

doact_2x1:      cpx #MAXSPR-1
                bcs doact_2x1skip
doact_2x1ok:    tay                             ;Check Y-coord high
                bne doact_2x1skip
                lda temp1                       ;Store coordinates
                sta sprxl,x
                adc #$18
                sta sprxl+1,x
                lda temp2
                sta sprxh,x
                adc #$00
                sta sprxh+1,x
                lda temp3
                sta spry,x
                sta spry+1,x
                lda temp7
                asl                             ;Get right frameindex
                tay
                lda (temp5),y
                sta sprf,x
                iny
                lda (temp5),y
                sta sprf+1,x
                inx
                inx
                stx numsprites
doact_2x1skip:  jmp doact_restx

doact_2x2:      sta temp4
                cpx #MAXSPR-3                   ;Enough free sprites?
                bcs doact_2x2totalskip
doact_2x2ok:    lda temp1                       ;Store coordinates
                sta sprxl,x
                sta sprxl+2,x
                adc #$18
                sta sprxl+1,x
                sta sprxl+3,x
                lda temp2
                sta sprxh,x
                sta sprxh+2,x
                adc #$00
                sta sprxh+1,x
                sta sprxh+3,x
                lda temp7
                asl                             ;Get right frameindex
                asl
                tay
                lda temp3
                sta spry,x
                sta spry+1,x
                adc #$15                        ;Carry is still clear :-)
                sta spry+2,x
                sta spry+3,x
                lda (temp5),y
                sta sprf,x
                iny
                lda (temp5),y
                sta sprf+1,x
                iny
                lda (temp5),y
                sta sprf+2,x
                iny
                lda (temp5),y
                sta sprf+3,x
                lda temp4
                bne doact_2x2totalskip
                inx
                inx
                adc #$00
                bne doact_2x2skip
                inx
                inx
doact_2x2skip:  stx numsprites
doact_2x2totalskip:
                jmp doact_restx

doact_3x2:      sta temp4
                cpx #MAXSPR-5
                bcs doact_2x2totalskip
                lda #$03
                sta doact_reloadxc+1
                sta doact_xc
                lda #$02
                sta doact_yc
                lda temp7
                asl                             ;Get right frameindex
                sta temp7
                asl
                adc temp7
                tay
                jmp doact_arbitrary

doact_4x3:      sta temp4
                cpx #MAXSPR-11
                bcs doact_2x2totalskip
                lda #$04
                sta doact_reloadxc+1
                sta doact_xc
                lda #$03
                sta doact_yc
                lda temp7
                asl                             ;Get right frameindex
                asl
                sta temp7
                asl
                adc temp7
                tay
                jmp doact_arbitrary

doact_4x2totalskip:jmp doact_restx
doact_4x2:      sta temp4
                cpx #MAXSPR-7
                bcs doact_4x2totalskip
                lda #$04
                sta doact_reloadxc+1
                sta doact_xc
                lda #$02
                sta doact_yc
                lda temp7
                asl                             ;Get right frameindex
                asl
                asl
                tay
                jmp doact_arbitrary

doact_arbitrary:
                lda temp1
                sta doact_reloadxl+1
                lda temp2
                sta doact_reloadxh+1
doact_sprloop:  lda temp4                       ;Skip whole row?
                bne doact_skiprow
doact_sprloop2: lda temp1
                sta sprxl,x
                clc
                adc #$18
                sta temp1
                lda temp2
                sta sprxh,x
                adc #$00
                sta temp2
                lda temp3
                sta spry,x
                lda (temp5),y
                iny
                sta sprf,x
                cmp #EMPTYSPRITE
                beq doact_sprskip
                inx
doact_sprskip:  dec doact_xc
                bne doact_sprloop2
doact_rowdone:  dec doact_yc
                beq doact_sprdone
doact_reloadxl: lda #$00
                sta temp1
doact_reloadxh: lda #$00
                sta temp2
doact_reloadxc: lda #$00
                sta doact_xc
                lda temp3
                clc
                adc #21
                sta temp3
                bcc doact_nospryh
                inc temp4
doact_nospryh:  jmp doact_sprloop
doact_skiprow:  iny
                dec doact_xc
                bne doact_skiprow
                beq doact_rowdone
doact_sprdone:  stx numsprites
                jmp doact_restx


;Ŀ
;MOVEACTORX                                                                   
;MOVEACTORXDIRECT                                                             
;                                                                             
;Does X-movement of an actor. MOVEACTORXDIRECT should be used with care       
;because it assumes that the flags reflect correctly the A register.          
;                                                                             
;Parameters: X: Actor number, A: Amount of move (only in MOVEACTORXDIRECT)    
;Returns: -                                                                   
;Modifies: A                                                                  
;

moveactorx:     lda actsx,x
moveactorxdirect:
                clc
                beq mactx_pnot
                bmi mactx_neg
mactx_pos:      adc actxl,x
                sta actxl,x
                bcc mactx_pnot
                inc actxh,x
mactx_pnot:     rts
mactx_neg:      adc actxl,x
                sta actxl,x
                bcs mactx_nnot
                dec actxh,x
mactx_nnot:     rts

;Ŀ
;MOVEACTORY                                                                   
;MOVEACTORYDIRECT                                                             
;                                                                             
;Does Y-movement of an actor. MOVEACTORYDIRECT should be used with care       
;because it assumes that the flags reflect correctly the A register.          
;                                                                             
;Parameters: X: Actor number, A: Amount of move (only in MOVEACTORYDIRECT)    
;Returns: -                                                                   
;Modifies: A                                                                  
;

moveactory:     lda actsy,x
moveactorydirect:
                clc
                beq macty_pnot
                bmi macty_neg
macty_pos:      adc actyl,x
                sta actyl,x
                bcc macty_pnot
                inc actyh,x
macty_pnot:     rts
macty_neg:      adc actyl,x
                sta actyl,x
                bcs macty_nnot
                dec actyh,x
macty_nnot:     rts

;Ŀ
;ACCACTORX                                                                    
;                                                                             
;Does X-acceleration of an actor. Note: acceleration and speed limit must     
;always be same sign!                                                         
;                                                                             
;Parameters: X: Actor number, A: acceleration, Y: Speed limit                 
;Returns: -                                                                   
;Modifies: A,Y,temp6                                                          
;

slowaccactorx:  clc
                sty temp6
                ldy sortsprstart
                beq aact_xok
                rts

accactorx:      clc
                sty temp6
aact_xok:       tay                             ;Just to get the flags
                bmi aact_xneg
aact_xpos:      adc actsx,x
                bmi aact_xposnl                 ;If speed still negative,
                cmp temp6                       ;need not check limit
                bcc aact_xposnl
                lda temp6
aact_xposnl:    sta actsx,x
                rts
aact_xneg:      adc actsx,x
                bpl aact_xnegnl
                cmp temp6
                bcs aact_xnegnl
                lda temp6
aact_xnegnl:    sta actsx,x
                rts

;Ŀ
;ACCACTORY                                                                    
;                                                                             
;Does Y-acceleration of an actor. Note: acceleration and speed limit must     
;always be same sign!                                                         
;                                                                             
;Parameters: X: Actor number, A: acceleration, Y: Speed limit                 
;Returns: -                                                                   
;Modifies: A,Y,temp6                                                          
;

slowaccactory:  clc
                sty temp6
                ldy sortsprstart
                beq aact_yok
                rts

accactory:      clc
                sty temp6
aact_yok:       tay                             ;Just to get the flags
                bmi aact_yneg
aact_ypos:      adc actsy,x
                sta actsy,x
                bmi aact_yposnl                 ;If speed still negative,
                cmp temp6                       ;need not check limit
                bcc aact_yposnl
                lda temp6
                sta actsy,x
aact_yposnl:    rts
aact_yneg:      adc actsy,x
                sta actsy,x
                bpl aact_ynegnl
                cmp temp6
                bcs aact_ynegnl
                lda temp6
                sta actsy,x
aact_ynegnl:    rts

;Ŀ
;BRAKEACTORX                                                                  
;                                                                             
;Brakes X-speed of an actor towards zero.                                     
;                                                                             
;Parameters: X: Actor number, A: deceleration (always positive)               
;Returns: -                                                                   
;Modifies: A,temp6                                                            
;

brakeactorx:    sta temp6
                lda actsx,x
                beq bact_xdone
                bmi bact_xneg
bact_xpos:      sec
                sbc temp6
                bpl bact_xdone
                lda #$00
bact_xdone:     sta actsx,x
                rts
bact_xneg:      clc
                adc temp6
                bmi bact_xdone
                lda #$00
                sta actsx,x
                rts

;Ŀ
;BRAKEACTORY                                                                  
;                                                                             
;Brakes Y-speed of an actor towards zero.                                     
;                                                                             
;Parameters: X: Actor number, A: deceleration (always positive)               
;Returns: -                                                                   
;Modifies: A,temp6                                                            
;

brakeactory:    sta temp6
                lda actsy,x
                beq bact_ydone
                bmi bact_yneg
bact_ypos:      sec
                sbc temp6
                bpl bact_ydone
                lda #$00
bact_ydone:     sta actsy,x
                rts
bact_yneg:      clc
                adc temp6
                bmi bact_ydone
                lda #$00
                sta actsy,x
                rts

;Ŀ
;FACEACTOR                                                                    
;                                                                             
;Faces actor X in the direction of actor Y                                    
;Note! Does not actually turn the actor, but returns the direction to which   
;it should be turned!                                                         
;                                                                             
;Parameters: X:Actor number, Y:Actor number                                   
;Returns: A:Direction                                                         
;Modifies: A                                                                  
;

faceactor:      lda actxl,y
                sec
                sbc actxl,x
                lda actxh,y
                sbc actxh,x
                and #$80
                rts

;Ŀ
;FACEACTORY                                                                   
;                                                                             
;Does the same as FACEACTOR, but to the Y coordinate.                         
;Note! Does not actually turn the actor, but returns the direction to which   
;it should be turned!                                                         
;                                                                             
;Parameters: X:Actor number, Y:Actor number                                   
;Returns: A:Direction                                                         
;Modifies: A,temp1-temp2                                                      
;

faceactory:     lda actyl,y
                sec
                sbc #$60                        ;12 pixels upward
                sta temp1
                lda actyh,y
                sbc #$00
                sta temp2
                lda temp1
                sbc actyl,x
                lda temp2
                sbc actyh,x
                and #$80
                rts


;Ŀ
;GETACTORXDIST                                                                
;                                                                             
;Gets absolute X-distance of actor X to actor Y (in blocks)                   
;                                                                             
;Parameters: X:Actor number, Y:Actor number                                   
;Returns: A:Distance                                                          
;Modifies: A                                                                  
;

getactorxdist:  lda actxl,y
                sec
                sbc actxl,x
                lda actxh,y
                sbc actxh,x
                bpl gaxd_ok
                lda actxl,x
                sec
                sbc actxl,y
                lda actxh,x
                sbc actxh,y
gaxd_ok:        rts

;Ŀ
;GETACTORYDIST                                                                
;                                                                             
;Gets absolute Y-distance of actor X to actor Y (in blocks)                   
;                                                                             
;Parameters: X:Actor number, Y:Actor number                                   
;Returns: A:Distance                                                          
;Modifies: A                                                                  
;

getactorydist:  lda actyl,y
                sec
                sbc actyl,x
                lda actyh,y
                sbc actyh,x
                bpl gayd_ok
                lda actyl,x
                sec
                sbc actyl,y
                lda actyh,x
                sbc actyh,y
gayd_ok:        rts

;Ŀ
;REVERSEACTOR                                                                 
;                                                                             
;Reverses actor direction.                                                    
;                                                                             
;Parameters: X: Actor number                                                  
;Returns: -                                                                   
;Modifies: A,temp6                                                            
;

reverseactor:   lda actd,x
                eor #$80
                sta actd,x
                rts

;Ŀ
;CHECKCOLLISION                                                               
;                                                                             
;Checks if two actors have collided.                                          
;Note: both actors must exist and their distance must not exceed 127 chars!   
;                                                                             
;Parameters: X: Actor number                                                  
;            Y: Actor number                                                  
;Returns: C=1 No collision, C=0 Actors collided (note: this order is unusual) 
;Modifies: A,temp1-temp5                                                      
;

checkcollision: stx temp1
                sty temp2
                lda actt,x              ;Get actor types
                tax                     ;(indexes into actor size table)
                lda actt,y
                tay
                lda atsizeh-1,x
                clc
                adc atsizeh-1,y
                sta temp3               ;Temp3 = Xsize1+Xsize2
                lda atsizeu-1,x
                adc atsized-1,y
                sta temp4               ;Temp4 = Ysize1+Ysize2
                lda atsized-1,x
                adc atsizeu-1,y
                sta temp5               ;Temp5 = Ysize1+Ysize2 other way
                ldx temp1
                jmp chkcol_common

;Ŀ
;CHECKPOINTCOLLISION                                                          
;                                                                             
;Checks if two actors have collided. The first actor is assumed to be point-  
;like, with zero X & Y size.                                                  
;Note: both actors must exist and their distance must not exceed 127 chars!   
;                                                                             
;Parameters: X: Actor number                                                  
;            Y: Actor number                                                  
;Returns: C=1 No collision, C=0 Actors collided (note: this order is unusual) 
;Modifies: A,temp1-temp5                                                      
;

checkpointcollision:
                sty temp2
                lda actt,y
                tay
                lda atsizeh-1,y
                sta temp3               ;Temp3 = Xsize2
                lda atsized-1,y
                sta temp4               ;Temp4 = Ysize2
                lda atsizeu-1,y
                sta temp5               ;Temp5 = Ysize2 other way

chkcol_common:  ldy temp2
                lda actxl,x             ;Subtract X-coords to get distance
                sec
                sbc actxl,y
                sta temp1
                lda actxh,x
                sbc actxh,y
                bpl chkcol_xpos
                cmp #$f9
                bcc chkcol_toofar
                eor #$ff
                tay
                lda temp1
                eor #$ff
                adc #$00                ;Add $01, Carry=1
                bcc chkcol_xnotover
                iny
chkcol_xnotover:lsr
                lsr
                lsr
                ora coordtbllo,y
                ldy temp2
                cmp temp3
                bcc chkcol_xok
                rts
chkcol_xpos:    cmp #$04
                bcs chkcol_toofar2
                tay
                lda temp1
                lsr
                lsr
                lsr
                ora coordtbllo,y
                ldy temp2
                cmp temp3
                bcc chkcol_xok
                rts
chkcol_toofar:  sec
chkcol_toofar2: rts
chkcol_xok:     lda actyl,x             ;Subtract Y-coords to get distance
                sec
                sbc actyl,y
                sta temp1
                lda actyh,x
                sbc actyh,y
                bpl chkcol_ypos
chkcol_yneg:    cmp #$f9
                bcc chkcol_toofar
                eor #$ff
                tay
                lda temp1
                eor #$ff
                adc #$00                ;Add $01, Carry=1
                bcc chkcol_ynotover
                iny
chkcol_ynotover:lsr
                lsr
                lsr
                ora coordtbllo,y
                ldy temp2
                cmp temp5               ;Y-distance
                rts
chkcol_ypos:    cmp #$07
                bcs chkcol_toofar2
                tay
                lda temp1
                lsr
                lsr
                lsr
                ora coordtbllo,y
                ldy temp2
                cmp temp4               ;Y-distance
                rts

;Ŀ
;CHECKLANDING                                                                 
;CHECKLANDING2                                                                
;                                                                             
;Checks if a groundbased actor who jumps, has landed.                         
;CHECKLANDING2 does not call GETCHARINFO but instead uses the value of temp3. 
;This is to make the "second" landingcheck for diagonal jumps (there must be  
;2 checks for 100% reliability) faster.                                       
;                                                                             
;Parameters: -                                                                
;Returns: C=0 no landing, C=1 landed                                          
;Modifies: A,Y                                                                
;

checklanding:   lda actsy,x
                bmi chkl_noground
                jsr getcharposinfo
                and #CI_GROUND
                beq chkl_noground
dolanding:      lda actyl,x                     ;Align Y on a char
                and #$c0
                sta actyl,x
                lda #$00
                sta actsy,x
                lda actbits,x
                and #$ff-AB_JUMP                ;Clear jump bit
                sta actbits,x
                sec
                rts
chkl_noground:  clc
                rts
checklanding2:  lda actsy,x
                bmi chkl_noground
                lda temp3
                and #CI_GROUND
                bne dolanding
                clc
                rts

checkheadbump:  lda #-4
                jsr getcharposinfooffset
                and #CI_OBSTACLE
                beq chkl_noheadbump
                lda #$00
                sta actsy,x
chkl_noheadbump:clc
                rts

;Ŀ
;CHECKSIDEOBST                                                                
;CHECKJUMPOBST                                                                
;                                                                             
;Performs X-move of actor but if an obstacle blocks the way, does no          
;movement. CHECKSIDEOBST checks one char higher.                              
;                                                                             
;Parameters: -                                                                
;Returns: temp3: result of GETCHARINFO, C=0 no obstacle, C=1 obstacle         
;Modifies: A,Y,temp1,temp2,temp3                                              
;

checksideobst:  lda actxl,x                     ;Store X-position
                sta temp1
                lda actxh,x
                sta temp2
                lda actsx,x                     ;If X-speed zero, do nothing
                beq cso_ok
                jsr moveactorxdirect
                lda #-1
                jsr getcharposinfooffset
cso_common:     sta temp3
                and #CI_OBSTACLE
                beq cso_ok
cso_restorepos: lda temp1                       ;Restore X-position before move
                sta actxl,x
                lda temp2
                sta actxh,x
                sec
                rts
cso_ok:         clc
                rts

checkjumpobst:  lda actxl,x                     ;Store X-position
                sta temp1
                lda actxh,x
                sta temp2
                lda actsx,x                     ;If X-speed zero, do nothing
                beq cso_ok
                jsr moveactorxdirect
                jsr getcharposinfo
                jmp cso_common

;Ŀ
;CHECKSTAIRS                                                                  
;                                                                             
;Checks if there is ground at player's position or above/below. Will move     
;player one char up or down, if necessary.                                    
;                                                                             
;Parameters: -                                                                
;Returns: C=0 no ground, C=1 there's ground                                   
;         temp3 = result of GETCHARINFO                                       
;Modifies: A,Y,temp3                                                          
;

checkstairs:    jsr getcharposinfo
                sta temp3
                and #CI_GROUND
                bne cst_ground
                lda #1
                jsr getcharposinfooffset
                sta temp3
                and #CI_GROUND
                bne cst_down
                lda #-1
                jsr getcharposinfooffset
                sta temp3
                and #CI_GROUND
                bne cst_up
                clc                             ;No ground
                rts
cst_down:       lda #64
                jsr moveactorydirect
cst_ground:     sec
                rts
cst_up:         lda #-64
                jsr moveactorydirect
                sec
                rts

checkstairs_preferup:
                lda #-1
                jsr getcharposinfooffset
                sta temp3
                and #CI_GROUND
                bne cst_up
                jsr getcharposinfo
                sta temp3
                and #CI_GROUND
                bne cst_ground
                lda #1
                jsr getcharposinfooffset
                sta temp3
                and #CI_GROUND
                bne cst_down
                clc                             ;No ground
                rts

;Ŀ
;GETFREEACTOR                                                                 
;                                                                             
;Searches for a free actor in the actor table.                                
;When a free actor is found, some things are resetted here                    
;(like actor bits, leveldata origin etc.)                                     
;                                                                             
;Parameters: A:Last actor to be checked                                       
;            Y:First actor to be checked                                      
;Returns: C=1 Found (Y:Index) C=0 Not found                                   
;Modifies: A,X,Y                                                              
;

getfreeactor:   sta gfa_cmp+1
gfa_loop:       lda actt,y
                beq gfa_found
gfa_cmp:        cpy #$00
                iny
                bcc gfa_loop
                clc
                rts
gfa_found:      sta acthp,y
                sta actf,y
                sta actfd,y
                sta actsx,y
                sta actsy,y
                sta actbits,y
                sta actattk,y                   ;Reset actor's attack
                sta actattkd,y                  ;variables
                lda #$ff
                sta actorg,y
                sec
                rts

