;ͻ
;Musicroutine 10 by Cadaver in May-June 2000                                  
;-------------------------------------------                                  
;                                                                             
;Patterndata features:                                                        
;- All 8 octaves, unlike in Sadotracker :-)                                   
;- Compact & fast-to-read patterndata, a bit like in Rob Hubbard's routine    
;  * A downside of the compactness is that you must use transpose to reach    
;    the highest octaves                                                      
;- Note durations up to 254, or 255 (infinite)                                
;- Rest & continue commands                                                   
;- Instrument change command                                                  
;- Slide command (toneportamento)                                             
;- Tie notes achieved by using the highest slide speed (127)                  
;- Max. 256 bytes in a pattern                                                
;                                                                             
;Instrument features:                                                         
;- Combined arpeggio & waveformtable                                          
;  * Both absolute & relative arpeggio                                        
;  * Compact 1 byte arpeggios also possible (for relative halftones 0-15)     
;- Table-based pulse effect                                                   
;- Table-based vibrato                                                        
;- All tables have jump/stop command                                          
;- Simple filtercontrol: channel 1 can set filtertype & cutoff,               
;  cutoff-sweep not supported                                                 
;- Max. 128 instruments                                                       
;- Max. 256 bytes in arpeggio table                                           
;- Max. 256 bytes in combined pulse/vibrato table                             
;                                                                             
;Songdata features:                                                           
;- Repeat command (up to 47 repeats of the same pattern :-) )                 
;- Transpose command (range -16 to +63 halftones)                             
;- Jump command (can jump even to different song :-))                         
;- Call & Return commands (these must not be nested :-))                      
;- Max. 31 songs                                                              
;- Max. 128 patterns per song                                                 
;- Unlimited song length                                                      
;                                                                             
;Sound FX features:                                                           
;- Sound effects on all 3 channels                                            
;  * When playing, a sound effect overrides the musical note on its channel.  
;    All music commands are processed during this -> no sync errors, and      
;    when the effect is finished, music continues on the next note.           
;- Sound FX priority (higher number = higher priority)                        
;- Max. 32 sound FX                                                           
;- Max. 256 bytes in a sound-FX "arpeggio"table                               
;                                                                             
;Other features & details:                                                    
;- Built-in fadeout with 96 different speeds                                  
;- Without sound FX, takes usually max. 18 lines rastertime                   
;- Needs 5 zeropage locations                                                 
;ͼ

                processor 6502
                org $4000

;Ŀ
;Zeropage locations used by the music routine                                 
;                                                                             
;The temp variables may be re-used in your raster routine, but mt_patttbllo   
;and mt_patttblhi must remain untouched.                                      
;

mt_patttbllo    = $fb
mt_patttblhi    = $fc
mt_temp1        = $fd
mt_temp2        = $fe
mt_temp3        = $ff

;Ŀ
;Music routine defines                                                        
;

MODE_PLAY       = $00
MODE_INITSONG   = $01
MODE_INITFADE   = $20
MODE_INITFX     = $80

TRANS           = $90
REPEAT          = $d0
RET             = $fd
CALL            = $fe
JUMP            = $ff

DUR             = $80
CMD             = $40
SLIDE           = $80
ENDPATT         = $ff

C0              = 0
CIS0            = 1
D0              = 2
DIS0            = 3
E0              = 4
F0              = 5
FIS0            = 6
G0              = 7
GIS0            = 8
A0              = 9
B0              = 10
H0              = 11
C1              = 12
CIS1            = 13
D1              = 14
DIS1            = 15
E1              = 16
F1              = 17
FIS1            = 18
G1              = 19
GIS1            = 20
A1              = 21
B1              = 22
H1              = 23
C2              = 24
CIS2            = 25
D2              = 26
DIS2            = 27
E2              = 28
F2              = 29
FIS2            = 30
G2              = 31
GIS2            = 32
A2              = 33
B2              = 34
H2              = 35
C3              = 36
CIS3            = 37
D3              = 38
DIS3            = 39
E3              = 40
F3              = 41
FIS3            = 42
G3              = 43
GIS3            = 44
A3              = 45
B3              = 46
H3              = 47
C4              = 48
CIS4            = 49
D4              = 50
DIS4            = 51
E4              = 52
F4              = 53
FIS4            = 54
G4              = 55
GIS4            = 56
A4              = 57
B4              = 58
H4              = 59
C5              = 60
CIS5            = 61
REST            = 62
CONT            = 63

;Ŀ
;Music routine code                                                           
;                                                                             
;Call music in your raster routine. Put commands to music+1:                  
;                                                                             
;00 - Resume playing (this happens automatically after the other commands)    
;01 - Init tune 1                                                             
;02 - Init tune 2 etc...                                                      
;20 - Fade music with the slowest speed                                       
;7f - Fade music with the fastest speed                                       
;80 - Play sound effect 1 on channel 1                                        
;81 - Play sound effect 1 on channel 2                                        
;82 - Play sound effect 1 on channel 3                                        
;84 - Play sound effect 2 on channel 1 etc...                                 
;

music:          lda #$01                ;Play or init music/fadeout/sound FX?
                beq mt_music
                bpl mt_init
                jmp mt_fxinit           ;Negative value = init FX

mt_init:        cmp #MODE_INITFADE      ;Init fadeout?
                bcs mt_initfadeout
                asl                     ;Multiply songnum by 8
                asl                     ;to get offset to songtable
                asl
                tay
                lda songtbl-8,y         ;Get address of pattern table
                sta mt_patttbllo
                lda songtbl-7,y
                sta mt_patttblhi
                ldx #42                 ;Clear some channel variables
                lda #$00
mt_initzeroloop:sta chn_counter-1,x
                dex
                bne mt_initzeroloop
mt_initloop:    iny
                iny
                lda songtbl-8,y         ;Load songpointers
                sta chn_songptrlo,x
                lda songtbl-7,y
                sta chn_songptrhi,x
                dec chn_pattindex,x     ;Patternindex=$ff
                lda #$03
                sta chn_counter,x       ;Counter=$03
                lda #$08
                sta chn_waveform,x      ;Waveform=$08 (testbit to reset noise)
                lda chn_nextchnnum,x    ;Go to next channel
                tax
                bne mt_initloop
                sta music+1             ;Set playmode
                sta mt_fadespd+1        ;Reset fadeout
                sta mt_filttype+1       ;Reset filter
                lda #$0f                ;Reset volume to full
                sta mt_volume+1
                rts

mt_initfadeout: sbc #MODE_INITFADE-1
                sta mt_fadespd+1        ;Set fadespeed
mt_initfadedone:lda #$00
                sta music+1             ;Set playmode

mt_music:       tax
mt_fadespd:     lda #$00                ;Fadeout routine
                beq mt_musicloop
                clc
mt_fadecnt:     adc #$00
                sta mt_fadecnt+1
                bcc mt_musicloop
                dec mt_volume+1
                bpl mt_musicloop
                stx mt_volume+1
                stx mt_fadespd+1        ;Stop fade when volume 0 reached

mt_musicloop:   lda chn_counter,x       ;Counter zero? (reload & init note)
                beq mt_loadcounter
                cmp #$02                ;Counter 2? (advance song-position)
                beq mt_checkpatt
                bcs mt_pulseeffect      ;Do all effects in normal operation
                jmp mt_getnewnote       ;Counter 1? (get new note)

mt_loadcounter: lda chn_duration,x
                sta chn_counter,x
                lda chn_initflag,x      ;If it was a rest or continued note,
                beq mt_pulseeffect      ;do all effects

mt_initnote:    ldy chn_sound,x         ;Get soundnumber
                lda snd_vibindex,y      ;Init vibrato
                sta chn_vibindex,x
                lda snd_arpindex,y      ;Init arpeggio
                sta chn_arpindex,x
                lda snd_pulseindex,y    ;Init pulse
                sta chn_pulseindex,x
                lda snd_pulse,y
                sta chn_pulselo,x
                sta chn_pulsehi,x
                txa                     ;Only channel 1 controls filter
                bne mt_nofiltinit
                lda snd_filtcutoff,y
                sta mt_filtcutoff+1     ;Cutoff MSB
                lda snd_filttype,y
                sta mt_filttype+1
mt_nofiltinit:  bpl mt_vibeffect

mt_checkpatt:   lda chn_pattindex,x
                cmp #$ff
                bne mt_vibeffect
                jmp mt_nextpatt

mt_pulseeffect: inx
                jsr mt_effect
                dex

mt_slideeffect: lda chn_slide,x
                bne mt_doslide

mt_vibeffect:   ldy chn_arpindex,x
                bne mt_arpeffect
                jsr mt_effect
                jmp mt_loadregs

mt_arpjump:     lda arptbl,y
                beq mt_arpdone          ;Jump to location zero stops arpeggio
                tay
mt_arpeffect:   lda arptbl-1,y          ;Get waveform or rel.arp
                cmp #$10
                bcc mt_relarp
                cmp #JUMP
                beq mt_arpjump
mt_arpwave:     sta chn_waveform,x
                iny
                lda arptbl-1,y          ;Get absolute or relative arp.
                bmi mt_absarp
mt_relarp:      adc chn_note,x
mt_absarp:      iny
                sty mt_temp3
                and #$7f
                tay
                lda freqtbllo,y         ;Load frequency
                sta chn_freqlo,x
                lda freqtblhi,y
                sta chn_freqhi,x
                lda mt_temp3
mt_arpdone:     sta chn_arpindex,x      ;Store new arpeggiotableindex
                jmp mt_loadregs

mt_tienote:     lda freqtbllo,y
                sta chn_freqlo,x
                lda freqtblhi,y         ;Frequency highbyte always nonzero
                bne mt_slideend2
mt_doslide:     ldy chn_note,x
                cmp #$fe                ;Tie note?
                beq mt_tienote
                sta mt_temp3
                lda freqtbllo,y
                sta mt_temp1
                lda freqtblhi,y
                sta mt_temp2
                jsr mt_16bitcompare     ;Go up or down?
                bcs mt_slidedown
mt_slideup:     jsr mt_frequp
mt_slupcheck:   jsr mt_16bitcompare
                bcs mt_slideend
                bcc mt_loadregs
mt_slidedown:   jsr mt_freqdown
mt_sldncheck:   jsr mt_16bitcompare
                bcs mt_loadregs
mt_slideend:    lda mt_temp1            ;Slide finished, copy
                sta chn_freqlo,x        ;destination freq to
                lda mt_temp2            ;channel freq
mt_slideend2:   sta chn_freqhi,x
                ldy chn_sound,x         ;Get soundnumber
                lda snd_vibindex,y      ;Init vibrato
                sta chn_vibindex,x
                lda #$00                ;Zero slidespeed = no slide
                sta chn_slide,x
                beq mt_loadregs

mt_loadregs:    ldy chn_fxstatus,x      ;FX playing?
                bne mt_dofx
                lda chn_pulselo,x       ;Load pulse, frequency, waveform
                sta $d402,x
                lda chn_pulsehi,x
                sta $d403,x
                lda chn_freqlo,x
                sta $d400,x
                lda chn_freqhi,x
                sta $d401,x
                lda chn_waveform,x
                sta $d404,x
                lda chn_initflag,x      ;Load ADSR if noteinitflag on
                beq mt_nextchn
                ldy chn_sound,x
                lda snd_ad,y
                sta $d405,x
                lda snd_sr,y
                sta $d406,x
                dec chn_initflag,x
mt_nextchn:     lda chn_counter,x       ;If counter has value $ff then
                cmp #$ff                ;don't decrement it at all
                sbc #$00                ;(infinite noteduration)
                sta chn_counter,x
mt_nodec:       lda chn_nextchnnum,x    ;Go to next channel
                tax
                beq mt_filtcutoff
                jmp mt_musicloop

mt_filtcutoff:  lda #$00
                sta $d416               ;Cutoff MSB
mt_filttype:    lda #$00
                tay
                ora #$f0                ;Full resonance
                sta $d417
                tya
                and #$f0
mt_volume:      ora #$0f                ;Add volume
                sta $d418
                rts

mt_dofx:        cpy #$02                ;Sound FX play or init?
                ldy chn_fxnumber,x
                bcc mt_fxgoing

mt_fxinit2:     lda #$00
                sta chn_fxindex,x       ;Reset tableindex
                sta chn_initflag,x      ;Reset noteinitflag
                sta $d402,x             ;Reset pulsewidth low
                sta $d404,x             ;Hard restart for FX
                sta $d405,x
                sta $d406,x
                lda fx_pulse,y
                sta $d403,x             ;Take pulsewidth high
                dec chn_fxstatus,x      ;Go to playmode next time
                jmp mt_nextchn

mt_fxgoing:     lda fx_tbllo,y
                sta mt_temp1
                lda fx_tblhi,y
                sta mt_temp2
                ldy chn_fxindex,x       ;First frame of sound?
                bne mt_fxnotfirst
                lda (mt_temp1),y
                ldy chn_fxnumber,x
                sta $d404,x             ;Load waveform
                lda fx_ad,y             ;Load AD
                sta $d405,x
                lda fx_sr,y             ;Load SR
                sta $d406,x
                ldy #$01
mt_fxnotfirst:  lda (mt_temp1),y        ;Take waveform or freq.
                bne mt_fxnotend         ;Zero ends fx
                sta chn_waveform,x
                sta chn_arpindex,x      ;The note on that channel
                sta chn_fxnumber,x      ;will be silent to its end
                sta chn_fxstatus,x
                jmp mt_nextchn

mt_fxnotend:    cmp #$82                ;Frequency or waveform?
                bcs mt_fxloadfreq2
                sta $d404,x             ;Load waveform
                iny
mt_fxloadfreq:  lda (mt_temp1),y        ;Get notenumber
mt_fxloadfreq2: iny
                sty mt_temp3
                tay
                lda freqtbllo-$80,y     ;Load frequency
                sta $d400,x
                lda freqtblhi-$80,y
                sta $d401,x
                lda mt_temp3
                sta chn_fxindex,x
                jmp mt_nextchn

mt_fxinit:      sta mt_temp3
                and #$03                ;Take the channel number
                tay
                ldx fxchntbl,y
                lda mt_temp3
                lsr                     ;Take the fx number
                lsr
                and #$1f
                cmp chn_fxnumber,x      ;Check fx priority
                bcc mt_fxtoolowpri
                sta chn_fxnumber,x      ;Save new priority
                lda #$02                ;Continue fxinit later
                sta chn_fxstatus,x
mt_fxtoolowpri: jmp mt_initfadedone     ;Return to normal music playing mode

mt_effect:      ldy chn_vibindex,x      ;This code used for both pulse
                beq mt_noeffect         ;& vibrato routine
                tya
                and #$01                ;If the table index is odd,
                beq mt_nonewtime        ;get new time
mt_newtime:     lda pulsevibtbl-1,y
                cmp #JUMP               ;Jump command?
                beq mt_effectjump
                sta chn_vibtime,x
                inc chn_vibindex,x
                rts
mt_effectjump:  lda pulsevibtbl,y       ;Jump to position 0 ends
                sta chn_vibindex,x      ;effect
                bne mt_effect
mt_noeffect:    rts
mt_nonewtime:   dec chn_vibtime,x       ;Decrease timecounter
                bne mt_timeleft
                inc chn_vibindex,x      ;If time zero, move table index to
mt_timeleft:    lda pulsevibtbl-1,y     ;next odd position
                tay                     ;(will get new time)
                asl
                sta mt_temp3
                tya
                bmi mt_freqdown

mt_frequp:      lda chn_freqlo,x        ;Subroutines for
                clc                     ;frequency up/down slides
                adc mt_temp3
                sta chn_freqlo,x
                bcc mt_frequpdone
                inc chn_freqhi,x
mt_frequpdone:  rts

mt_freqdown:    lda chn_freqlo,x
                sec
                sbc mt_temp3
                sta chn_freqlo,x
                bcs mt_freqdowndone
                dec chn_freqhi,x
mt_freqdowndone:rts

mt_16bitcompare:lda chn_freqhi,x        ;Subroutine for 16-bit
                cmp mt_temp2            ;frequency compare
                bne mt_16bitdone
mt_checklo:     lda chn_freqlo,x
                cmp mt_temp1
mt_16bitdone:   rts

mt_getnewnote:  lda #$00                ;Assume:no slide
                sta chn_slide,x
                ldy chn_pattnum,x
                lda (mt_patttbllo),y
                sta mt_temp1
                iny
                lda (mt_patttbllo),y
                sta mt_temp2
                ldy chn_pattindex,x
                lda (mt_temp1),y
                sta mt_temp3            ;Store note
                bpl mt_nodur            ;Check 7th bit: Duration command?
                iny
                lda (mt_temp1),y
                sta chn_duration,x      ;Store duration
mt_nodur:       bit mt_temp3            ;Check 6th bit: Slide or Snd command?
                bvc mt_noteok
                iny
                lda (mt_temp1),y
                bmi mt_setslide
mt_setsnd:      sta chn_sound,x
                bpl mt_noteok           ;Sound number always positive
mt_setslide:    asl
                sta chn_slide,x         ;Save slidespeed
mt_noteok:      iny
                lda (mt_temp1),y        ;Check end of pattern
                cmp #$ff
                beq mt_endpatt
                tya
mt_endpatt:     sta chn_pattindex,x
                lda mt_temp3            ;Check note
                and #$3f
                cmp #REST
                beq mt_rest             ;Is it rest or continue?
                bcs mt_cont
                adc chn_transpose,x
                sta chn_note,x
                lda chn_slide,x         ;If slide, don't perform hardrestart
                bne mt_cont
                ldy chn_fxstatus,x      ;If FX playing, no hardrestart
                beq mt_hardrestart
                jmp mt_dofx
mt_hardrestart: sta $d404,x
                sta $d405,x
                sta $d406,x
                inc chn_initflag,x
                jmp mt_nextchn
mt_rest:        lda chn_waveform,x
                and #$fe
                sta chn_waveform,x
mt_cont:        jmp mt_vibeffect

mt_nextpatt:    lda #$00                ;Reset patternindex
                sta chn_pattindex,x
                tay
                lda chn_songptrlo,x     ;Get songpointer
                sta mt_temp1
                lda chn_songptrhi,x
                sta mt_temp2
mt_getpattern:  lda (mt_temp1),y        ;Get next pattern
                bpl mt_pattok
                iny
                cmp #REPEAT
                bcc mt_settrans
                cmp #JUMP
                beq mt_songjump
                cmp #RET
                bcc mt_setrepeat
                bne mt_songcall
mt_songret:     lda chn_returnlo,x
                sta mt_temp1
                lda chn_returnhi,x
                sta mt_temp2
                ldy #$01
                bne mt_getpattern
mt_songcall:    tya
                adc mt_temp1
                sta chn_returnlo,x
                lda mt_temp2
                adc #$00
                sta chn_returnhi,x
                bne mt_songjump
mt_setrepeat:   sbc #REPEAT             ;Carry=0, subtract one more
                sta chn_repeat,x        ;on purpose
                bne mt_getpattern
mt_settrans:    sbc #TRANS-1            ;Carry=0
                sta chn_transpose,x     ;Set transpose
                jmp mt_getpattern
mt_songjump:    lda (mt_temp1),y        ;Get position where to jump
                pha
                iny
                lda (mt_temp1),y
                sta mt_temp2
                pla
                sta mt_temp1
                ldy #$00
                beq mt_getpattern
mt_pattok:      asl
                sta chn_pattnum,x       ;Save patternnumber
                lda chn_repeat,x        ;Repeats left?
                beq mt_norepeat
                dey
                dec chn_repeat,x
mt_norepeat:    iny
                tya
                clc
                adc mt_temp1
                sta chn_songptrlo,x
                lda mt_temp2
                adc #$00
                sta chn_songptrhi,x     ;Songptr highbyte always nonzero
                jmp mt_vibeffect        ;Do only vibrato/arpeggio to save time

;Ŀ
;Music routine variables and tables                                           
;

freqtbllo:      dc.b $19,$29,$3b,$4e,$62,$77,$8d,$a5,$be,$d8,$f4,$12
                dc.b $32,$53,$76,$9c,$c4,$ee,$1a,$4a,$7c,$b1,$e9,$24
                dc.b $64,$a6,$ed,$38,$88,$dc,$35,$94,$f8,$62,$d2,$49
                dc.b $c8,$4d,$db,$71,$10,$b8,$6b,$28,$f0,$c4,$a5,$93
                dc.b $90,$9b,$b6,$e2,$20,$71,$d6,$50,$e0,$89,$4a,$27
                dc.b $20,$36,$6d,$c5,$41,$e2,$ac,$a0,$c1,$12,$95,$4e
                dc.b $40,$6d,$da,$8a,$82,$c5,$59,$41,$83,$25,$2b,$9d
                dc.b $80,$da,$b4,$15,$04,$8b,$b2,$83,$07,$4a,$57,$ff

freqtblhi:      dc.b $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$02
                dc.b $02,$02,$02,$02,$02,$02,$03,$03,$03,$03,$03,$04
                dc.b $04,$04,$04,$05,$05,$05,$06,$06,$06,$07,$07,$08
                dc.b $08,$09,$09,$0a,$0b,$0b,$0c,$0d,$0d,$0e,$0f,$10
                dc.b $11,$12,$13,$14,$16,$17,$18,$1a,$1b,$1d,$1f,$21
                dc.b $23,$25,$27,$29,$2c,$2e,$31,$34,$37,$3b,$3e,$42
                dc.b $46,$4a,$4e,$53,$58,$5d,$63,$69,$6f,$76,$7d,$84
                dc.b $8c,$94,$9d,$a7,$b1,$bb,$c6,$d2,$df,$ec,$fa,$ff

chn_counter:    dc.b 0           ;This is the first variable that is zeroed
chn_repeat:     dc.b 0
chn_pattindex:  dc.b 0
chn_vibindex:   dc.b 0
chn_pulseindex: dc.b 0
chn_arpindex:   dc.b 0
chn_slide:      dc.b 0

                dc.b 0,0,0,0,0,0,0
                dc.b 0,0,0,0,0,0,0

chn_waveform:   dc.b 0
chn_songptrlo:  dc.b 0
chn_songptrhi:  dc.b 0
chn_transpose:  dc.b 0
chn_sound:      dc.b 0
chn_duration:   dc.b 0
chn_initflag:   dc.b 0          ;This is the last variable that is zeroed

                dc.b 0,0,0,0,0,0,0
                dc.b 0,0,0,0,0,0,0

chn_note:       dc.b 0
chn_freqlo:     dc.b 0
chn_pulselo:    dc.b 0
chn_freqhi:     dc.b 0
chn_pulsehi:    dc.b 0
chn_vibtime:    dc.b 0
chn_pulsetime:  dc.b 0

                dc.b 0,0,0,0,0,0,0
                dc.b 0,0,0,0,0,0,0

chn_pattnum:    dc.b 0
chn_fxstatus:   dc.b 0
chn_fxnumber:   dc.b 0
chn_fxindex:    dc.b 0
chn_nextchnnum: dc.b 7
chn_returnlo:   dc.b 0
chn_returnhi:   dc.b 0

                dc.b 0,0,0,0,14,0,0
                dc.b 0,0,0,0,0,0,0

fxchntbl:       dc.b 0, 7, 14

;Ŀ
;Music data!                                                                  
;

                include mus_sfx.s
                include mus_data.s

                include mus_sng0.s
                include mus_sng1.s
                include mus_sng2.s
                include mus_sng3.s
                include mus_sng4.s
                include mus_sng5.s
                include mus_sng6.s
                include mus_sng7.s
                include mus_sng8.s
                include mus_sng9.s
                include mus_sn10.s
                include mus_sn11.s
                include mus_sn12.s
                include mus_sn13.s
                include mus_sn14.s
                include mus_sn15.s

songtbl:        dc.w song0patttbl
                dc.w song0chn1
                dc.w song0chn2
                dc.w song0chn3

                dc.w song1patttbl
                dc.w song1chn1
                dc.w song1chn2
                dc.w song1chn3

                dc.w song2patttbl
                dc.w song2chn1
                dc.w song2chn2
                dc.w song2chn3

                dc.w song3patttbl
                dc.w song3chn1
                dc.w song3chn2
                dc.w song3chn3

                dc.w song4patttbl
                dc.w song4chn1
                dc.w song4chn2
                dc.w song4chn3

                dc.w song5patttbl
                dc.w song5chn1
                dc.w song5chn2
                dc.w song5chn3

                dc.w song6patttbl
                dc.w song6chn1
                dc.w song6chn2
                dc.w song6chn3

                dc.w song7patttbl
                dc.w song7chn1
                dc.w song7chn2
                dc.w song7chn3

                dc.w song8patttbl
                dc.w song8chn1
                dc.w song8chn2
                dc.w song8chn3

                dc.w song9patttbl
                dc.w song9chn1
                dc.w song9chn2
                dc.w song9chn3

                dc.w song10patttbl
                dc.w song10chn1
                dc.w song10chn2
                dc.w song10chn3

                dc.w song11patttbl
                dc.w song11chn1
                dc.w song11chn2
                dc.w song11chn3

                dc.w song12patttbl
                dc.w song12chn1
                dc.w song12chn2
                dc.w song12chn3

                dc.w song13patttbl
                dc.w song13chn1
                dc.w song13chn2
                dc.w song13chn3

                dc.w song14patttbl
                dc.w song14chn1
                dc.w song14chn2
                dc.w song14chn3

                dc.w song15patttbl
                dc.w song15chn1
                dc.w song15chn2
                dc.w song15chn3

