;ͻ
;Metal Warrior III by Cadaver: Init routines (that will be overwritten)       
;Started in 5/2000!                                                           
;ͼ

                org initcodestart

                sei
                cld
                ldx #$ff
                txs
                jsr initbank
                jsr initraster
                jsr initsprites                 ;Init sprite-multiplexing
                jsr cleartextscreen2            ;Clear screen
                jsr drawpanel
                jsr initvideo
                jsr initcontrols
                jmp start



;Ŀ
;INITBANK                                                                     
;                                                                             
;Clears memory from $c000-$ffff and copies textscreen characters & common     
;sprites in the place. This is called in the very beginning.                  
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

initbank:       lda #$20                                ;Blank the screen
                sta $d011
                lda #$34
                sta $01
                lda #$c0
                sta ib_sta+2
                lda #$00
                tay
ib_sta:         sta gamechars,y
                iny
                bne ib_sta
                inc ib_sta+2
                bne ib_sta
                lda #>ib_textchars
                sta ib_lda2+2
                lda #>textchars
                sta ib_sta2+2
ib_lda2:        lda ib_textchars,y
ib_sta2:        sta textchars,y
                iny
                bne ib_lda2
                inc ib_lda2+2
                inc ib_sta2+2
                lda ib_sta2+2
                cmp #>(textchars+$b00)
                bcc ib_lda2
                rts

;Ŀ
;INITRASTER                                                                   
;                                                                             
;Initializes cycling of raster interrupts and switches KERNAL off.            
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

initraster:     lda #$35                        ;Set all ROM off, IO on.
                sta $01
                lda #<idleirq                   ;Set an idle-IRQ which will
                sta $0314                       ;be used when KERNAL is on
                lda #>idleirq
                sta $0315
                lda #<nmi                       ;Set NMI, both the hardware
                sta $0318                       ;vector and the vector used by
                sta $fffa                       ;KERNAL
                lda #>nmi
                sta $0319
                sta $fffb
                lda #<raster_gscreen            ;Set IRQ vector for first
                sta $fffe                       ;raster interrupt
                lda #>raster_gscreen
                sta $ffff
                lda #RPOS_GSCREEN               ;Set low bits of raster
                sta $d012                       ;position for first interrupt
                lda #$00                        ;Set high bit of raster
                sta $d011                       ;position (0)
                lda #$7f                        ;Set timer interrupt off
                sta $dc0d
                lda #$01                        ;Set raster interrupt on
                sta $d01a
                lda $dc0d                       ;Acknowledge timer interrupt
                lda #MUSIC_SILENCE              ;Set music to silence
                sta music+1
                rts

;Ŀ
;INITSPRITES                                                                  
;                                                                             
;Initializes the sprite-IRQ & sprite-doublebuffer system.                     
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

initsprites:    lda #$00
                sta rgscr_nonewspr+1
                sta irqsprstart
                sta irqirqstart
                sta newsprflag
                rts

;Ŀ
;INITVIDEO                                                                    
;                                                                             
;Sets all necessary VIC registers and enables interrupts.                     
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

initvideo:      lda #$00                        ;The only true border color!
                sta screen                      ;Set gamescreen number
                sta $d020
                sta $d021

                sta $d015                       ;Sprites off
                sta $d01d                       ;Sprite X-expand off
                sta $d017                       ;Sprite Y-expand off
                lda #$ff                        ;All sprites multicolor
                sta $d01c
                lda $dd00                       ;Set game videobank
                and #$fc
                sta $dd00
                cli
                rts

;Ŀ
;INITCONTROLS                                                                 
;                                                                             
;Initializes control zeropage variables.                                      
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

initcontrols:   lda #$00
                sta joystick
                sta prevjoy
                lda #$ff
                sta keypress
                sta keytype
                rts

;Ŀ
;DRAWPANEL                                                                    
;                                                                             
;Draws score panel to bottom of screen.                                       
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

drawpanel:      jsr initgame
                ldx #$00
                ldy #$00
drp_loop:       inx
                lda paneldata-1,x
                beq drp_end
                bpl drp_char
drp_string:     sta temp1
                inx
drp_stringloop: jsr drp_putchar
                dec temp1
                bmi drp_stringloop
                bpl drp_loop
drp_char:       jsr drp_putchar
                bne drp_loop
drp_putchar:    lda paneldata-1,x
                cmp #20
                bcc drp_putspace
                sta textscreen+SCROLLROWS*40,y
                lda #$0b
drp_cont:       sta colors+SCROLLROWS*40,y
                iny
                rts
drp_putspace:   pha
                lda #$20
                sta textscreen+SCROLLROWS*40,y
                pla
                bpl drp_cont
drp_end:        jsr removemessage2
                jsr updatepanel
                jsr updatepanel
                jmp updatepanel

paneldata:      dc.b 24,128+7,36,25,128+19,36,25,128+7,36,26
                dc.b 27,128+7,1,27,128+19,1,27,128+3,11,1,128+2,7,27
                dc.b 27,128+7,7,27,128+19,1,27,128+3,11,1,128+2,7,27
                dc.b 27,128+7,1,27,128+19,1,27,128+4,1,128+2,7,27
                dc.b 28,128+7,29,30,128+19,29,30,128+7,29,31
                dc.b 0

ib_textchars:   incbin bg\text.chr
                incbin spr\common.spr

