;ͻ
;Metal Warrior III by Cadaver: Inventory/Panel routines                       
;Started in 7/2000!                                                           
;ͼ

MSG_OFF         = 0
MSG_READY       = 1
MSG_WRITING     = 2

;Ŀ
;Item-specific use routines                                                   
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

usefood:        lda #MAXPLRHEALTH/4
                bne usehealth_common

usebeer:        lda #MAXPLRHEALTH/6
                bne usehealth_common

usemedikit:     lda #MAXPLRHEALTH/2-8           ;Restore almost 1/2 of health rapidly
                ldy acthp+ACTI_PLAYER           ;Medikit can't be used if
                cpy #MAXPLRHEALTH               ;already at full health
                bcs skipmedikit
usehealth_common:
                ldy teammember
                sta tmhealspd,y
                lda #SFX_POWERUP
                jsr playsfx
useitem_common: ldy invselect
                lda #$01
                jsr decreaseammo
                jmp preventattack

preventattack:  ldy invselect
                lda invtype,y
                cmp #ITEM_PISTOL                ;If not using a melee weapon,
                bcc prvattk_nogun               ;give a small delay to prevent
                cmp #ITEM_LASTWEAPON+1
                bcs prvattk_nogun
                lda #$10                        ;unwanted attacks by holding
                sta actattkd+ACTI_PLAYER        ;fire button down
prvattk_nogun:
skipmedikit:    rts

;Ŀ
;INITINVENTORY                                                                
;                                                                             
;Initializes player's inventory at game start                                 
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X                                                                
;

initinventory:  lda #$00
                sta meleebreak
                sta invselect
                sta invview
                ldx #INVENTORYSIZE-1
initinv_loop:   sta invtype,x
                sta invammolo,x
                sta invammohi,x
                dex
                bpl initinv_loop
                lda #ITEM_FISTS                 ;Fists are always with the
                sta invtype                     ;player :-)
                jmp setupdateflag

;Ŀ
;SORTINVENTORY                                                                
;                                                                             
;Sorts items to ascending order. Uses horrible bubblesort :-)                 
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,temp6                                                            
;

sortinventory:  stx sortinvrestx+1
                sty sortinvresty+1
                ldx #$00
sortinvloop1:   lda invtype+1,x                 ;When outer loop reaches the
                bne sortinvnotdone              ;end, we're done
sortinvrestx:   ldx #$00
sortinvresty:   ldy #$00
                rts
sortinvnotdone: txa
                tay                             ;Inner loop begins one row
                iny                             ;lower
sortinvloop2:   lda invtype,y
                beq sortinvinnerend
                cmp invtype,x
                bcc sortinvswap
sortinvswapdone:iny
                bne sortinvloop2
sortinvinnerend:inx
                bne sortinvloop1

sortinvswap:    cpx invselect                   ;If currently selected
                bne sortinvnosswap1             ;item must be swapped we
                sty invselect                   ;follow it
                beq sortinvsswap
sortinvnosswap1:cpy invselect
                bne sortinvnosswap2
                stx invselect
sortinvsswap:   jsr checkinvview
sortinvnosswap2:lda invtype,x
                pha
                lda invtype,y
                sta invtype,x
                pla
                sta invtype,y
                lda invammolo,x
                pha
                lda invammolo,y
                sta invammolo,x
                pla
                sta invammolo,y
                lda invammohi,x
                pha
                lda invammohi,y
                sta invammohi,x
                pla
                sta invammohi,y
                jmp sortinvswapdone

;Ŀ
;FINDITEM                                                                     
;                                                                             
;Tries to find an item of certain type from the player's inventory            
;                                                                             
;Parameters: A:Item type                                                      
;Returns: C=1 item found (Y:Inventory index), C=0 not found                   
;Modifies: Y                                                                  
;

finditem:       ldy #INVENTORYSIZE-1
fi_loop:        cmp invtype,y
                beq fi_found                    ;Carry=1
                dey
                bpl fi_loop
                clc
fi_found:       rts

;Ŀ
;ADDAMMO                                                                      
;                                                                             
;Adds ammo to item                                                            
;                                                                             
;Parameters: A:Amount to add in BCD format                                    
;            Y:Inventory index                                                
;Returns: A:Amount that was really added                                      
;Modifies: A,Y,temp1-temp3                                                    
;

addammo:        sta temp1
                stx temp3
                ldx invtype,y
                sed
                lda itemmaxtbllo,x
                sec
                sbc invammolo,y
                sta temp2                       ;How much we can add
                lda itemmaxtblhi,x
                sbc invammohi,y
                bne addammo_all                 ;If highbyte > 0, we can add
                lda temp2                       ;the whole amount
                cmp temp1
                bcs addammo_all
                sta temp1
addammo_all:    lda invammolo,y
                clc
                adc temp1
                sta invammolo,y
                lda invammohi,y
                adc #$00
                sta invammohi,y
                cld
                lda temp1
                ldx temp3
                rts

;Ŀ
;DECREASEAMMO                                                                 
;                                                                             
;Decreases ammo of item                                                       
;Note: does not check below zero condition!                                   
;                                                                             
;Parameters: A:Amount of decrease in BCD format                               
;            Y:Inventory index                                                
;Returns: -                                                                   
;Modifies: A,Y (if item must be removed),temp1                                
;

decreaseammo:   sta temp1
                sed
                lda invammolo,y
                sec
                sbc temp1
                sta invammolo,y
                lda invammohi,y
                sbc #$00
                sta invammohi,y
                cld
                ora invammolo,y
                bne decammo_notzero
                lda invtype,y
                cmp #ITEM_PISTOL                ;Weapons are not removed
                bcc decammo_remove              ;when ammo runs out because
                cmp #ITEM_LASTWEAPON+1          ;they can be reloaded
                bcc decammo_dontremove
removeitem:
decammo_remove: sty temp1
rmitem_loop:    lda invtype+1,y
                sta invtype,y
                lda invammolo+1,y
                sta invammolo,y
                lda invammohi+1,y
                sta invammohi,y
                iny
                cpy #INVENTORYSIZE
                bcc rmitem_loop
                ldy temp1
                cpy invselect                   ;Is it the selected item?
                beq decammo_selected
                bcs decammo_notselected
decammo_selected:
                jmp itemsel_up2
decammo_notselected:
decammo_dontremove:
decammo_notzero:jmp setupdateflag

;Ŀ
;AR_ITEM                                                                      
;                                                                             
;Item's actormovement routine                                                 
;                                                                             
;Parameters: X:Actor number                                                   
;Returns: -                                                                   
;Modifies: -                                                                  
;

ar_item:        lda actf,x
                cmp #ITEM_BOOTDISK              ;Some items are always
                bcs ar_itemdone                 ;stationary
                lda actbits,x                   ;Item flying?
                bne ar_itemflying
ar_itemdone:    rts
ar_itemflying:  lda #1
                ldy #3*8
                jsr accactory
                lda actsy,x
                sta temp1
                jsr moveactorydirect
                jsr checklanding
                bcc ar_itemdone
ar_itemlanded:  lda temp1
                clc
                eor #$ff
                adc #$06
                bpl ar_itemdone
                sta actsy,x
                lda #AB_JUMP
                sta actbits,x
                rts

;Ŀ
;CHECKPICKUP                                                                  
;                                                                             
;Checks for player picking up items                                           
;                                                                             
;Parameters: X:Player actor number                                            
;Returns: -                                                                   
;Modifies: A,Y,temp1-temp3,temp7                                              
;

checkpickup:    stx cpu_skip+1
                ldy #ACTI_FIRSTITEM
cpu_loop:       lda actt,y              ;This really is an item?
                cmp #ACT_ITEM
                beq cpu_righttype
cpu_next:       cpy #ACTI_LASTITEM
                iny                     ;Go to next item until all checked
                bcc cpu_loop
                rts
cpu_righttype:  jsr checkcollision
                bcs cpu_next            ;No collision
                sty temp3               ;Store Y
                lda actf,y              ;Get item type
                tax
                cmp #ITEM_CREDITS       ;Credits?
                bne cpu_nocredits
                jsr random              ;A random amount
                and #$03
                tay
                lda creditstbl,y
                bne cpu_credits
cpu_nocredits:  lda itemaddtbl,x        ;Get item amount
cpu_credits:    jsr additem
                bcc cpu_skip
                lda #SFX_SELECT
                jsr playsfx
                ldy temp3
                lda #$00                ;Remove item from ground
                sta actt,y
                lda itemnametbllo,x
                sta cpu_lda+1
                lda itemnametblhi,x
                sta cpu_lda+2
                ldy #$00                ;Print "picked up" text
cpu_lda:        lda $1000,y
                beq cpu_nameend
                sta pickeduptext+10,y
                iny
                bne cpu_lda
cpu_nameend:    cpy #ITEMNAMELENGTH
                bcs cpu_namedone
                lda #$20
cpu_nameendloop:sta pickeduptext+10,y
                iny
                bne cpu_nameend
cpu_namedone:   ldy #20
                lda #<pickeduptext
                ldx #>pickeduptext
                jsr newmessage
cpu_skip:       ldx #$00
                rts

;Ŀ
;ADDITEM                                                                      
;                                                                             
;Adds an item to player inventory                                             
;                                                                             
;Parameters: A: Amount in BCD format (0-99)                                   
;            X: Item type                                                     
;Returns: C=1 Add successful, C=0 inventory full or max.amount carried        
;Modifies: A,Y,temp1-temp2                                                    
;

additem:        stx temp1
                sta temp2
                ldy #$00                ;Search empty space or same item from
ai_search:      lda invtype,y           ;inventory
                beq ai_found
                cmp temp1
                beq ai_sameitem
                iny
                cpy #INVENTORYSIZE
                bcc ai_search
ai_notok:       clc                     ;Inventory full
                rts
ai_found:       txa
                sta invtype,y
                lda itemmaxtblhi,x
                bne ai_nolimit
                lda itemmaxtbllo,x
                cmp temp2
                bcc ai_limit
ai_nolimit:     lda temp2
ai_limit:       sta invammolo,y
ai_ok:          jsr sortinventory
                jsr setupdateflag
                ldx temp1
                sec                     ;Add succeeded
                rts
ai_sameitem:    lda itemammotbl,x       ;Does the item have ammo (like a pistol
                beq ai_noammo           ;etc.)
                tax
                lda temp2               ;Then add to the ammo storage instead
                jmp additem
ai_noammo:      jsr ai_checkmax
                bcc ai_notfull
                clc                     ;Max.amount already
                rts
ai_notfull:     sed
                lda invammolo,y
                clc
                adc temp2
                sta invammolo,y
                lda invammohi,y
                adc #$00
                sta invammohi,y
                cld
                jsr ai_checkmax
                bcc ai_ok
                lda itemmaxtbllo,x
                sta invammolo,y
                lda itemmaxtblhi,x
                sta invammohi,y
                jmp ai_ok

ai_checkmax:    lda invammohi,y
                cmp itemmaxtblhi,x
                beq ai_checkmaxlo
                rts
ai_checkmaxlo:  lda invammolo,y
                cmp itemmaxtbllo,x
                rts

;Ŀ
;SELECTITEM                                                                   
;                                                                             
;Item selection with ,. or SPACE keys.                                        
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X                                                                
;

selectitem:     lda actattk+ACTI_PLAYER         ;Must be in non-attacking
                ora actattkd+ACTI_PLAYER        ;position before selected
                beq itemsel_ok                  ;item can be changed
                rts
itemsel_ok:     lda keytype
                cmp #KEY_SPACE
                beq itemsel_nextweapon
                cmp #KEY_PILKKU
                beq itemsel_up
                cmp #KEY_PISTE
                beq itemsel_down
itemsel_uplimit:rts
itemsel_up:     lda invselect
                beq itemsel_uplimit
itemsel_up2:    dec invselect
itemsel_done:   jsr checkinvview
                jmp setupdateflag
itemsel_downlimit:rts
itemsel_down:   ldx invselect
                inx
                lda invtype,x
                beq itemsel_downlimit
                stx invselect
                jmp itemsel_done
itemsel_nextweapon:
                ldx invselect
itemsel_nextweaponloop:
                inx
itemsel_nextweaponloop2:
                stx invselect
                lda invtype,x                   ;Reached end of inventory?
                bne itemsel_nextweaponok
                ldx #$00                        ;Start from beginning
                beq itemsel_nextweaponloop2
itemsel_nextweaponok:
                cmp #ITEM_FIRSTWEAPON
                bcc itemsel_nextweaponloop
                cmp #ITEM_GRENADES              ;Melee weapon
                bcc itemsel_done
                cmp #ITEM_LASTWEAPON+1
                bcs itemsel_nextweaponloop
                lda invammolo,x                 ;Other than melee weapon;
                ora invammohi,x                 ;check that there is ammo
                bne itemsel_done
                beq itemsel_nextweaponloop

checkinvview:   lda invselect
                sec
                sbc invview
                bpl chkinvv_nottoolow
chkinvv_toolow: lda invselect
                bpl chkinvv_ok
chkinvv_nottoolow:
                cmp #$03
                bcc chkinvv_nottoohigh
chkinvv_toohigh:lda invselect
                sbc #$02
chkinvv_ok:     sta invview
chkinvv_nottoohigh:
                rts

;Ŀ
;ITEMCAPACITY                                                                 
;                                                                             
;Calculates maximum carrying capacity of the team. It changes when a member   
;enters or leaves the team, or when someone's strength changes.               
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y,temp1                                                        
;

itemcapacity:   ldx #MAXTM-1
                lda #$00
itcp_loop1:     ldy tmstat,x            ;Strength of all joined teammembers
                beq itcp_skip1          ;is the multiplier for item carrying
                clc                     ;capacity
                adc tmstrength,x
itcp_skip1:     dex
                bpl itcp_loop1
                sta temp1
                ldx #ITEM_GRENADES
                jsr itcp_calc
                ldx #ITEM_9MMBULLETS
itcp_loop:      jsr itcp_calc
                inx
                cpx #ITEM_BATTERIES+1
                bcc itcp_loop
 ;               lsr temp1               ;Halve the capacity for medikits
                ldx #ITEM_MEDIKIT
                jsr itcp_calc
                jmp setupdateflag

itcp_calc:      lda #$00
                sta itemmaxtbllo,x
                sta itemmaxtblhi,x
                ldy temp1
itcp_calcloop:  sed
                lda itemmaxtbllo,x
                clc
                adc itemaddtbl,x
                sta itemmaxtbllo,x
                lda itemmaxtblhi,x
                adc #$00
                sta itemmaxtblhi,x
                cld
                dey
                bne itcp_calcloop
                txa
                jsr finditem                    ;Check if there is too much
                bcc itcp_nottoomuch             ;of the item
                lda invammohi,y
                cmp itemmaxtblhi,x
                bcc itcp_nottoomuch
                bne itcp_toomuch
                lda invammolo,y
                cmp itemmaxtbllo,x
                bcc itcp_nottoomuch
itcp_toomuch:   lda itemmaxtbllo,x
                sta invammolo,y
                lda itemmaxtblhi,x
                sta invammohi,y
itcp_nottoomuch:rts


;Ŀ
;NEWMESSAGE                                                                   
;                                                                             
;Writes a message to the message/inventory screen.                            
;                                                                             
;Parameters: A:Message address low                                            
;            X:Message address high                                           
;            Y:Message time ($ff = infinite)                                  
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

newmessage:     sta msgptrlo
                stx msgptrhi
                sty msgtime
newmessage2:    jsr clearmsgscreen
                lda #MSG_WRITING
                sta msgstatus
                lda #$00
                sta msgcolumn
                sta msgrow
                sta msgpackedword
                rts

;Ŀ
;INSTANTMESSAGE                                                               
;                                                                             
;Writes the message to the message/inventory screen instantly.                
;Takes a lot of time so should not be called in the game loop :-)             
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y,temp1-temp4,temp7                                            
;

instantmessage: lda #60
                sta temp7
imsg_loop:      jsr updatepanel
                dec temp7
                bne imsg_loop
                rts

;Ŀ
;REMOVEMESSAGE                                                                
;                                                                             
;Removes the message from the message/inventory screen.                       
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

removemessage:  jsr clearmsgscreen
removemessage2: lda #MSG_OFF
                sta msgstatus
setupdateflag:  lda #$03
                sta invupdateflag
                rts

;Ŀ
;UPDATEPANEL                                                                  
;                                                                             
;Draws changed score, health etc.                                             
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

updatepanel:    ldx #$03
updp_loop1:     lda score,x                     ;Score changed?
                cmp prevscore,x
                beq updp_ok1
                sta prevscore,x
                ldy digitpostbl,x
                jsr updp_drawdigits
updp_ok1:       dex
                bpl updp_loop1
                lda acthp+ACTI_PLAYER           ;Hitpoints changed?
                cmp prevhp
                beq updp_ok2
                sta prevhp
                lsr                             ;Player max.health = 64
                lsr
                ldy #81
                ldx #89
                jsr updp_drawmeter
updp_ok2:       lda teammember                  ;Teammember changed?
                cmp prevtm
                beq updp_ok3
                sta prevtm
                asl
                asl
                tax
                stx textscreen+SCROLLROWS*40+32+40
                inx
                stx textscreen+SCROLLROWS*40+33+40
                inx
                stx textscreen+SCROLLROWS*40+32+80
                inx
                stx textscreen+SCROLLROWS*40+33+80
updp_ok3:       ldy teammember
                lda tmagility,y                 ;Agility changed?
                cmp prevagility
                beq updp_ok4
                sta prevagility
                ldy #76
                ldx #79
                jsr updp_drawmeter
                lda #"A"
                sta textscreen+SCROLLROWS*40+35+40
updp_ok4:       ldy teammember
                lda tmstrength,y                ;Strength changed?
                cmp prevstrength
                beq updp_ok5
                sta prevstrength
                ldy #116
                ldx #119
                jsr updp_drawmeter
                lda #"S"
                sta textscreen+SCROLLROWS*40+35+80
updp_ok5:       ldy teammember
                lda tmvitality,y                ;Vitality changed?
                cmp prevvitality
                beq updp_ok6
                sta prevvitality
                ldy #156
                ldx #159
                jsr updp_drawmeter
                lda #"V"
                sta textscreen+SCROLLROWS*40+35+120
updp_ok6:       ldx teammember
                lda tmexplo,x                   ;Experience points changed?
                cmp prevexp
                beq updp_ok7
                sta prevexp
                ldy #152
                jsr updp_drawdigits
updp_ok7:       lda tmexphi,x
                cmp prevexp+1
                beq updp_ok8
                sta prevexp+1
                ldy #150
                jsr updp_drawdigit
updp_ok8:       lda timemin                     ;Time changed?
                cmp prevtimemin
                beq updp_ok9
                sta prevtimemin
                ldy #127
                jsr updp_drawdigits
updp_ok9:       lda timehour
                cmp prevtimehour
                beq updp_ok10
                sta prevtimehour
                ldy #124
                jsr updp_drawdigits
updp_ok10:      ldy daynumber
                cpy prevdaynumber
                beq updp_ok11
                sty prevdaynumber
                lda dayletter1tbl,y
                sta textscreen+SCROLLROWS*40+1+120
                lda dayletter2tbl,y
                sta textscreen+SCROLLROWS*40+2+120
                lda dayletter3tbl,y
                sta textscreen+SCROLLROWS*40+3+120
                lda #":"
                sta textscreen+SCROLLROWS*40+6+120
updp_ok11:      jmp updatemessage

;Ŀ
;UPDATEMESSAGE                                                                
;                                                                             
;Updates message/inventory screen                                             
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

updmsg_packedword:
                sta updmsg_packedwordcmp+1
                jsr updmsg_nextletter
                lda msgptrlo
                sta updmsg_restmsgptrlo+1
                lda msgptrhi
                sta updmsg_restmsgptrhi+1
                lda #>mostusedword0
                sta msgptrhi
                lda #<mostusedword0
                ldy #$60
updmsg_packedwordcmp:
                cpy #$00
                beq updmsg_packedwordfound
                adc wordlengthtbl-$60,y
                iny
                bcc updmsg_packedwordcmp
                inc msgptrhi
                bne updmsg_packedwordcmp
updmsg_packedwordfound:
                sta msgptrlo
                lda wordlengthtbl-$60,y
                sta msgpackedword
                bne updmsg_write

updatemessage:  lda msgstatus
                beq updmsg_nothing
                cmp #MSG_WRITING
                bcs updmsg_write
                lda msgtime             ;Message time $ff is "infinite"
                cmp #$ff
                sbc #$00
                sta msgtime
                bne updmsg_notzero
                jmp removemessage
updmsg_nothing: ldx invupdateflag       ;Inventory update?
                bne updateinventory
updmsg_notzero: rts
updmsg_write:   ldy #$00
                ldx msgrow
                lda screenrowtbllo,x
                clc
                adc msgcolumn
                tax
                lda (msgptrlo),y
                beq updmsg_ready
                cmp #96
                bcs updmsg_packedword
                cmp #$21
                bcc updmsg_nextword
                sta textscreen+SCROLLROWS*40+10+40,x
updmsg_columnok:inc msgcolumn
updmsg_nextletter:
                inc msgptrlo
                bne updmsg_nextletter2
                inc msgptrhi
updmsg_nextletter2:
                lda msgpackedword
                beq updmsg_nodecrement
                dec msgpackedword
                bne updmsg_countnotzero
updmsg_restmsgptrlo:
                lda #$00
                sta msgptrlo
updmsg_restmsgptrhi:
                lda #$00
                sta msgptrhi
updmsg_nodecrement:
updmsg_countnotzero:
                rts
updmsg_ready:   lda msgpackedword
                bne updmsg_restmsgptrlo
                lda #MSG_READY
                sta msgstatus
                rts
updmsg_nextword:lda msgcolumn
updmsg_nextwordloop:
                iny                             ;Count the length of the
                pha
                lda (msgptrlo),y                ;next word
                tax
                pla
                cpx #$21
                bcc updmsg_nextworddone
                cpx #96
                bcc updmsg_nextwordnotpacked
                clc
                adc wordlengthtbl-$60,x
                bne updmsg_nextwordloop
updmsg_nextwordnotpacked:
                adc #$01
                bne updmsg_nextwordloop
updmsg_nextworddone:
                cmp #$14                        ;Will it go over the message
                bcc updmsg_columnok             ;screen border?
                lda #$00
                sta msgcolumn
                inc msgrow
                jmp updmsg_nextletter

;Ŀ
;UPDATEINVENTORY                                                              
;                                                                             
;Updates inventory screen                                                     
;                                                                             
;Parameters: X:row to be updated+1                                            
;Returns: -                                                                   
;Modifies: A,X,Y,temp1-temp4                                                  
;

updateinventory:dex                             ;Screen row number
                stx invupdateflag
                txa
                clc
                adc invview                     ;Inventory row number
                tay
                lda screenrowtbllo,x
                tax
                lda #$20
                cpy invselect                  ;Draw the select char?
                bne updinvnoselect
                lda #93
updinvnoselect: sta textscreen+SCROLLROWS*40+10+40,x
                lda invtype,y                  ;Take item type
                sty temp1
                tay
                lda itemnametbllo,y             ;Item name address
                sta updinvwritename+1
                lda itemnametblhi,y
                sta updinvwritename+2
                ldy #$00
updinvwritename:
                lda $1000,y
                beq updinvnameend
                sta textscreen+SCROLLROWS*40+11+40,x
                inx
                iny
                bne updinvwritename
updinvnameend:  lda #$20
updinvnameendloop:
                cpy #ITEMNAMELENGTH
                bcs updinvnamedone
                sta textscreen+SCROLLROWS*40+11+40,x
                inx
                iny
                bne updinvnameendloop
updinvnamedone: ldy temp1
                lda invammolo,y
                ora invammohi,y              ;If amount is 0 then don't
                beq updinvzeroamount           ;write it
                lda invammohi,y
                and #$0f
                ora #$30
                sta textscreen+SCROLLROWS*40+12+40,x
                lda invammolo,y
                lsr
                lsr
                lsr
                lsr
                ora #$30
                sta textscreen+SCROLLROWS*40+13+40,x
                lda invammolo,y
                and #$0f
                ora #$30
                sta textscreen+SCROLLROWS*40+14+40,x
                bne updinvnext
updinvzeroamount:
                lda #$20
                sta textscreen+SCROLLROWS*40+40+12,x
                sta textscreen+SCROLLROWS*40+40+13,x
                sta textscreen+SCROLLROWS*40+40+14,x
updinvnext:     rts

clearmsgscreen: ldx #19
                lda #$20
clrmsg_loop:    sta textscreen+SCROLLROWS*40+40+10,x
                sta textscreen+SCROLLROWS*40+80+10,x
                sta textscreen+SCROLLROWS*40+120+10,x
                dex
                bpl clrmsg_loop
                rts

updp_drawdigits:pha
                lsr
                lsr
                lsr
                lsr
                ora #$30
updp_drawcharanddigit:
                sta textscreen+SCROLLROWS*40,y
                pla
updp_drawdigit: and #$0f
                ora #$30
                sta textscreen+SCROLLROWS*40+1,y
                rts

updp_drawmeter: stx updp_meter3+1
                sta temp6
                lsr                             ;Get number of "full" chars
                tax
                beq updp_meter2
                lda #94
updp_meter1:    sta textscreen+SCROLLROWS*40,y
                iny
                dex
                bne updp_meter1
updp_meter2:    lda temp6                       ;Is there a "half" char?
                and #$01
                beq updp_meter3
                lda #95
                sta textscreen+SCROLLROWS*40,y
                iny
updp_meter3:    cpy #89                         ;Then fill the rest with space
                bcs updp_meter4
                lda #32
                sta textscreen+SCROLLROWS*40,y
                iny
                bne updp_meter3
updp_meter4:    rts

