;ͻ
;Metal Warrior III Level + high level loading routines                        
;Started in 7/2000!                                                           
;ͼ

;Ŀ
;LOADLEVEL                                                                    
;                                                                             
;Loads the current game map (level), and correct blocks & sprites for that    
;map.                                                                         
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

loadlevel:      lda levelnum
                cmp loadedlevel
                beq lm_done
                sta loadedlevel
                clc
                adc #FIRSTMAPFILE
                jsr loadfile
                lda #<lvlscript                         ;Run level-specific
                ldx #>lvlscript                         ;script
                jsr runscript
                ldy levelnum
                lda levelsprtbl,y
                jmp loadsprites

;Ŀ
;LOADSPRITES                                                                  
;                                                                             
;Loads enemy/NPC sprites.                                                     
;                                                                             
;Parameters: A:Spritefile number                                              
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

loadsprites:    cmp loadedsprites
                beq ls_done
                sta loadedsprites
loadsprites_common:
                clc
                adc #FIRSTSPRITEFILE
                jsr loadfile
                jmp initspritecols
lm_done:
ls_done:
ltm_done:
plt_done:       rts

;Ŀ
;LOADTEAMMEMBER                                                               
;                                                                             
;Ensures that correct teammember sprites are loaded. Called when switching    
;characters                                                                   
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

loadteammember: ldy teammember
                cpy loadedtm
                beq ltm_done
                sty loadedtm
                iny
                tya
                bne loadsprites_common

;Ŀ
;LOADFILE                                                                     
;                                                                             
;The high-level load routine. Will display an error message on error, wait    
;for button press and retry until successful.                                 
;                                                                             
;Parameters: A:File number                                                    
;Returns: -                                                                   
;Modifies: A,X,Y,temp1-temp4                                                  
;

loadfile:       sta load_filenum+1
                lda #$00                        ;Make sure sprites are switched
                sta rgscr_nonewspr+1            ;off
load_again:     jsr waitpanel2
load_filenum:   lda #$00
                jsr loader_loadfile
                bcc load_ok
load_error:     lda #<loaderrortext
                ldx #>loaderrortext
                ldy #$ff
                jsr newmessage
load_wait:      jsr waitandupdate
                jsr getfireclick
                bcc load_wait
                jsr removemessage
                jmp load_again
load_ok:        jmp initblocks                  ;Block table is always
                                                ;corrupted by the loader

;Ŀ
;PLAYLEVELTUNE                                                                
;                                                                             
;Plays music associated with current level.                                   
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y,many temp variables if music has to be loaded                
;

playleveltune:  ldy levelnum
                lda leveltunetbl,y
                cmp tunenumber
                beq plt_done
                jmp playtune

;Ŀ
;ADDALLACTORS                                                                 
;                                                                             
;Adds actors from the leveldata to the game. Checks every actor.              
;(to be used after entering a door etc.)                                      
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y,temp1                                                        
;

addallactors:   jsr sl_calcspritesub
                ldx #$00
                stx lvlactstart                 ;Go through all the actors
                stx bossfight                   ;Reset bossfight status
                stx prevbossfight
                lda #NUMLEVELACT
                sta addact_cmp+1                ;in leveldata
                jmp addact_common

;Ŀ
;ADDSOMEACTORS                                                                
;                                                                             
;Adds actors from the leveldata to the game. Only 16 actors will be checked   
;at one time to save rastertime! :-)                                          
;This routine calculates also the border values that will be used for actor   
;removal checking, so this must be called before MOVEACTORS!                  
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y,temp1                                                        
;

addsomeactors:  lda lvlactstart
                tax
                clc
                adc #$10                        ;16 actors at a time
                sta addact_cmp+1
                and #NUMLEVELACT-1
                sta lvlactstart                 ;Advance index for next time
addact_common:  lda charsubxh
                sec
                sbc #$01
                bcs addact_ok1
                lda #$00
addact_ok1:     sta actleftlimit                ;Left border
                lda charsubxh
                clc
                adc #$0b
                bcc addact_ok2
                lda #$ff
addact_ok2:     sta actrightlimit               ;Right border
                lda charsubyh
                sec
                sbc #$01
                bcs addact_ok3
                lda #$00
addact_ok3:     sta acttoplimit                 ;Top border
                lda charsubyh
                clc
                adc #$07
                bpl addact_ok4
                lda #$7f
addact_ok4:     sta actbottomlimit              ;Bottom border
                lda #$00
                sta generatornumber
addact_loop:    lda lvlactt,x                   ;Actor exists?
                bne addact_notzero
addact_next:    inx
addact_cmp:     cpx #$00
                bne addact_loop
                jmp enemygenerator
addact_notzero: lda lvlactx,x
                cmp actleftlimit
                bcc addact_next
                cmp actrightlimit
                bcs addact_next
addact_xok:     lda lvlacty,x
                cmp acttoplimit
                bcc addact_next
                cmp actbottomlimit
                bcs addact_next
addact_ok:      lda lvlactt,x                   ;Item or other?
                bpl addact_nogenerator
                sta generatornumber
                bne addact_next
addact_nogenerator:
                cmp #ACT_ITEM
                bne addact_other
                jmp addact_item
addact_other:   lda #ACTI_LASTOTHER
                ldy #ACTI_FIRSTOTHER
                jsr getfreeactor
                bcc addact_next                 ;Found a free actor?
                lda lvlactt,x
                cmp #ACT_SHOPKEEPER
                beq addact_itemok
                cmp #ACT_LASTENEMY+1
                bcs addact_zerohp
                lda lvlactp,x                   ;Store hitpoints
                bne addact_nodefault            ;Zero = use default
                sty temp1
                ldy lvlactt,x
                lda enemydefaulthp-ACT_FIRSTENEMY,y ;Zero hitpoints (can't shoot)
                ldy temp1                       ;for all others than enemies
addact_nodefault:
                sta acthp,y
addact_zerohp:
addact_position:lda lvlactx,x                   ;Init position
                sta actxh,y
                lda lvlacty,x
                sta actyh,y
                lda lvlactb,x
                and #$f0
                sta actyl,y
                lda lvlactb,x
                asl
                asl
                asl
                asl
                sta actxl,y
                lda actxh+ACTI_PLAYER           ;Face player
                sec
                sbc actxh,y
                and #$80
                sta actd,y
                stx temp1
                sty temp2
                ldx temp2
                jsr getcharposinfo
                and #CI_GROUND
                eor #AB_JUMP                    ;If no ground, set jumping bit
                ldx temp1
                ldy temp2
                sta actbits,y
                txa                             ;Store leveldata origin
                sta actorg,y
                lda lvlactt,x
                sta actt,y
                lda #$00                        ;Remove actor from leveldata
                sta lvlactt,x
                jmp addact_next
addact_item:    lda #ACTI_LASTITEM
                ldy #ACTI_FIRSTITEM
                jsr getfreeactor
                bcs addact_itemok               ;Found a free actor?
                jmp addact_next
addact_itemok:  lda lvlactp,x                   ;Store item type to frame
                sta actf,y
                bpl addact_position

;Ŀ
;ENEMYGENERATOR                                                               
;                                                                             
;Enemygenerator routine. Generators must be placed at 12 blocks intervals     
;(horizontally) on the map.                                                   
;                                                                             
;Parameters: A:Generator type                                                 
;Returns: -                                                                   
;Modifies: A,X,Y,temp1-temp3                                                  
;

enemygenerator: lda generatornumber
                bmi addact_gnoskip
                rts
addact_gnoskip: asl                             ;Index to generatortable
                asl
                asl
                sta temp2
                jsr random                      ;Which enemy of four possible?
                sta temp3
                and #$03
                asl
                adc temp2
                tay
                lda generatortbl,y              ;Get enemy type
                sta temp2
                lda generatortbl+1,y            ;Index to probability table
                adc colormode                   ;(depends on time)
                tay
                lda temp3
                cmp probabilitytbl,y            ;Should the enemy appear?
                bcs addact_gskip
                lda #ACTI_LASTOTHER             ;Use only 3 actors for
                ldy #ACTI_FIRSTOTHER+3          ;generated enemies
                jsr getfreeactor
                bcc addact_gskip
                tya                             ;Transfer actor number to X
                tax
                jsr random
                sta temp3                       ;Random Y position
                and #$03
                sec
                adc charsubyh
                sta actyh,x
                lda #$00
                sta actyl,x
                lda charsubxh
                sbc #$00                        ;Subtract $01, carry = 0
                asl temp3                       ;Random X position:
                bcc addact_gleft                ;left or right
                adc #$0a                        ;Add $0b, carry = 1
addact_gleft:   sta actxh,x
                lda #$80
                sta actxl,x
                lda #-1                         ;One char above must be
                jsr getcharposinfooffset        ;free of obstacles
                and #CI_OBSTACLE
                bne addact_gskip
                jsr getcharposinfo
                ldy temp2
                and #CI_GROUND+CI_OBSTACLE
                cmp enemygroundcheck-ACT_FIRSTENEMY,y
                bne addact_gskip
                tya
                sta actt,x
                cpy #ACT_LASTENEMY+1
                bcs addact_gzerohp
                lda enemydefaulthp-ACT_FIRSTENEMY,y ;Set hitpoints
                sta acthp,x
addact_gzerohp: ldy #ACTI_PLAYER                ;Face player
                jsr faceactor
                sta actd,x
addact_gskip:   rts

;Ŀ
;CREATEACTOR                                                                  
;                                                                             
;Creates an actor to the leveldata.                                           
;(to be used after entering a door etc.)                                      
;                                                                             
;Parameters: temp1,temp2,temp3,temp4,temp5                                    
;Returns: C=1 success C=0 failure                                             
;Modifies: A,X,Y                                                              
;

createactor:    jsr createactor_search
                bcc cract_skip
                lda temp1
                sta lvlactx,y
                lda temp2
                sta lvlacty,y
                lda temp3
                sta lvlactt,y
                lda temp4
                sta lvlactp,y
                lda temp5
                sta lvlactb,y
cract_skip:     rts

createactor_search:
                ldy #NUMLEVELACT-1
                sec
cract_loop:     lda lvlactt,y
                beq cract_found
                dey
                bpl cract_loop
                clc
cract_found:    rts
