;ͻ
;Musicroutine 10 (NO SOUNDEFFECTS version) by Cadaver in May-July 2000        
;---------------------------------------------------------------------        
;                                                                             
;Patterndata features:                                                        
;- All 8 octaves, unlike in Sadotracker :-)                                   
;- Compact & fast-to-read patterndata, a bit like in Rob Hubbard's routine    
;  * A downside of the compactness is that you must use transpose to reach    
;    the highest octaves                                                      
;- Note durations up to 254, or 255 (infinite)                                
;- Rest & continue commands                                                   
;- Instrument change command                                                  
;- Slide command (toneportamento)                                             
;- Tie notes achieved by using the highest slide speed (127)                  
;- Max. 256 bytes in a pattern                                                
;                                                                             
;Instrument features:                                                         
;- Combined arpeggio & waveformtable                                          
;  * Both absolute & relative arpeggio                                        
;  * Compact 1 byte arpeggios also possible (for relative halftones 0-15)     
;- Table-based pulse effect                                                   
;- Table-based vibrato                                                        
;- All tables have jump/stop command                                          
;- Simple filtercontrol: channel 1 can set filtertype & cutoff,               
;  cutoff-sweep not supported                                                 
;- Max. 128 instruments                                                       
;- Max. 256 bytes in arpeggio table                                           
;- Max. 256 bytes in combined pulse/vibrato table                             
;                                                                             
;Songdata features:                                                           
;- Repeat command (up to 47 repeats of the same pattern :-) )                 
;- Transpose command (range -16 to +63 halftones)                             
;- Jump command (can jump even to different song :-))                         
;- Call & Return commands (these must not be nested :-))                      
;- Max. 31 songs                                                              
;- Max. 128 patterns per song                                                 
;- Unlimited song length                                                      
;                                                                             
;Other features & details:                                                    
;- Built-in fadeout with 96 different speeds                                  
;- Takes usually max. 18 lines rastertime                                     
;- Needs 5 zeropage locations                                                 
;ͼ

                processor 6502
                org $c000

;Ŀ
;Zeropage locations used by the music routine                                 
;                                                                             
;The temp variables may be re-used in your raster routine, but mt_patttbllo   
;and mt_patttblhi must remain untouched.                                      
;

mt_patttbllo    = $fb
mt_patttblhi    = $fc
mt_temp1        = $fd
mt_temp2        = $fe
mt_temp3        = $ff

;Ŀ
;Music routine defines                                                        
;

MODE_PLAY       = $00
MODE_INITSONG   = $01
MODE_INITFADE   = $20

TRANS           = $90
REPEAT          = $d0
RET             = $fd
CALL            = $fe
JUMP            = $ff

DUR             = $80
CMD             = $40
SLIDE           = $80
ENDPATT         = $ff

C0              = 0
CIS0            = 1
D0              = 2
DIS0            = 3
E0              = 4
F0              = 5
FIS0            = 6
G0              = 7
GIS0            = 8
A0              = 9
B0              = 10
H0              = 11
C1              = 12
CIS1            = 13
D1              = 14
DIS1            = 15
E1              = 16
F1              = 17
FIS1            = 18
G1              = 19
GIS1            = 20
A1              = 21
B1              = 22
H1              = 23
C2              = 24
CIS2            = 25
D2              = 26
DIS2            = 27
E2              = 28
F2              = 29
FIS2            = 30
G2              = 31
GIS2            = 32
A2              = 33
B2              = 34
H2              = 35
C3              = 36
CIS3            = 37
D3              = 38
DIS3            = 39
E3              = 40
F3              = 41
FIS3            = 42
G3              = 43
GIS3            = 44
A3              = 45
B3              = 46
H3              = 47
C4              = 48
CIS4            = 49
D4              = 50
DIS4            = 51
E4              = 52
F4              = 53
FIS4            = 54
G4              = 55
GIS4            = 56
A4              = 57
B4              = 58
H4              = 59
C5              = 60
CIS5            = 61
REST            = 62
CONT            = 63

;Ŀ
;Music routine code                                                           
;                                                                             
;Call music in your raster routine. Put commands to music+1:                  
;                                                                             
;00 - Resume playing (this happens automatically after the other commands)    
;01 - Init tune 1                                                             
;02 - Init tune 2 etc...                                                      
;20 - Fade music with the slowest speed                                       
;7f - Fade music with the fastest speed                                       
;

music:          lda #$01                ;Play or init music/fadeout?
                beq mt_music
                bpl mt_init

mt_init:        cmp #MODE_INITFADE      ;Init fadeout?
                bcs mt_initfadeout
                asl                     ;Multiply songnum by 8
                asl                     ;to get offset to songtable
                asl
                tay
                lda songtbl-8,y         ;Get address of pattern table
                sta mt_patttbllo
                lda songtbl-7,y
                sta mt_patttblhi
                ldx #42                 ;Clear some channel variables
                lda #$00
mt_initzeroloop:sta chn_counter-1,x
                dex
                bne mt_initzeroloop
mt_initloop:    iny
                iny
                lda songtbl-8,y         ;Load songpointers
                sta chn_songptrlo,x
                lda songtbl-7,y
                sta chn_songptrhi,x
                dec chn_pattindex,x     ;Patternindex=$ff
                lda #$03
                sta chn_counter,x       ;Counter=$03
                lda #$08
                sta chn_waveform,x      ;Waveform=$08 (testbit to reset noise)
                lda chn_nextchnnum,x    ;Go to next channel
                tax
                bne mt_initloop
                sta music+1             ;Set playmode
                sta mt_fadespd+1        ;Reset fadeout
                sta mt_filttype+1       ;Reset filter
                lda #$0f                ;Reset volume to full
                sta mt_volume+1
                rts

mt_initfadeout: sbc #MODE_INITFADE-1
                sta mt_fadespd+1        ;Set fadespeed
mt_initfadedone:lda #$00
                sta music+1             ;Set playmode

mt_music:       tax
mt_fadespd:     lda #$00                ;Fadeout routine
                beq mt_musicloop
                clc
mt_fadecnt:     adc #$00
                sta mt_fadecnt+1
                bcc mt_musicloop
                dec mt_volume+1
                bpl mt_musicloop
                stx mt_volume+1
                stx mt_fadespd+1        ;Stop fade when volume 0 reached

mt_musicloop:   lda chn_counter,x       ;Counter zero? (reload & init note)
                beq mt_loadcounter
                cmp #$02                ;Counter 2? (advance song-position)
                beq mt_checkpatt
                bcs mt_pulseeffect      ;Do all effects in normal operation
                jmp mt_getnewnote       ;Counter 1? (get new note)

mt_loadcounter: lda chn_duration,x
                sta chn_counter,x
                lda chn_initflag,x      ;If it was a rest or continued note,
                beq mt_pulseeffect      ;do all effects

mt_initnote:    ldy chn_sound,x         ;Get soundnumber
                lda snd_vibindex,y      ;Init vibrato
                sta chn_vibindex,x
                lda snd_arpindex,y      ;Init arpeggio
                sta chn_arpindex,x
                lda snd_pulseindex,y    ;Init pulse
                sta chn_pulseindex,x
                lda snd_pulse,y
                sta chn_pulselo,x
                sta chn_pulsehi,x
                txa                     ;Only channel 1 controls filter
                bne mt_nofiltinit
                lda snd_filtcutoff,y
                sta mt_filtcutoff+1     ;Cutoff MSB
                lda snd_filttype,y
                sta mt_filttype+1
mt_nofiltinit:  bpl mt_vibeffect

mt_checkpatt:   lda chn_pattindex,x
                cmp #$ff
                bne mt_vibeffect
                jmp mt_nextpatt

mt_pulseeffect: inx
                jsr mt_effect
                dex

mt_slideeffect: lda chn_slide,x
                bne mt_doslide

mt_vibeffect:   ldy chn_arpindex,x
                bne mt_arpeffect
                jsr mt_effect
                jmp mt_loadregs

mt_arpjump:     lda arptbl,y
                beq mt_arpdone          ;Jump to location zero stops arpeggio
                tay
mt_arpeffect:   lda arptbl-1,y          ;Get waveform or rel.arp
                cmp #$10
                bcc mt_relarp
                cmp #JUMP
                beq mt_arpjump
mt_arpwave:     sta chn_waveform,x
                iny
                lda arptbl-1,y          ;Get absolute or relative arp.
                bmi mt_absarp
mt_relarp:      adc chn_note,x
mt_absarp:      iny
                sty mt_temp3
                and #$7f
                tay
                lda freqtbllo,y         ;Load frequency
                sta chn_freqlo,x
                lda freqtblhi,y
                sta chn_freqhi,x
                lda mt_temp3
mt_arpdone:     sta chn_arpindex,x      ;Store new arpeggiotableindex
                jmp mt_loadregs

mt_tienote:     lda freqtbllo,y
                sta chn_freqlo,x
                lda freqtblhi,y         ;Frequency highbyte always nonzero
                bne mt_slideend2
mt_doslide:     ldy chn_note,x
                cmp #$fe                ;Tie note?
                beq mt_tienote
                sta mt_temp3
                lda freqtbllo,y
                sta mt_temp1
                lda freqtblhi,y
                sta mt_temp2
                jsr mt_16bitcompare     ;Go up or down?
                bcs mt_slidedown
mt_slideup:     jsr mt_frequp
mt_slupcheck:   jsr mt_16bitcompare
                bcs mt_slideend
                bcc mt_loadregs
mt_slidedown:   jsr mt_freqdown
mt_sldncheck:   jsr mt_16bitcompare
                bcs mt_loadregs
mt_slideend:    lda mt_temp1            ;Slide finished, copy
                sta chn_freqlo,x        ;destination freq to
                lda mt_temp2            ;channel freq
mt_slideend2:   sta chn_freqhi,x
                ldy chn_sound,x         ;Get soundnumber
                lda snd_vibindex,y      ;Init vibrato
                sta chn_vibindex,x
                lda #$00                ;Zero slidespeed = no slide
                sta chn_slide,x
                beq mt_loadregs

mt_loadregs:    lda chn_pulselo,x       ;Load pulse, frequency, waveform
                sta $d402,x
                lda chn_pulsehi,x
                sta $d403,x
                lda chn_freqlo,x
                sta $d400,x
                lda chn_freqhi,x
                sta $d401,x
                lda chn_waveform,x
                sta $d404,x
                lda chn_initflag,x      ;Load ADSR if noteinitflag on
                beq mt_nextchn
                ldy chn_sound,x
                lda snd_ad,y
                sta $d405,x
                lda snd_sr,y
                sta $d406,x
                dec chn_initflag,x
mt_nextchn:     lda chn_counter,x       ;If counter has value $ff then
                cmp #$ff                ;don't decrement it at all
                sbc #$00                ;(infinite noteduration)
                sta chn_counter,x
mt_nodec:       lda chn_nextchnnum,x    ;Go to next channel
                tax
                beq mt_filtcutoff
                jmp mt_musicloop

mt_filtcutoff:  lda #$00
                sta $d416               ;Cutoff MSB
mt_filttype:    lda #$00
                tay
                ora #$f0                ;Full resonance
                sta $d417
                tya
                and #$f0
mt_volume:      ora #$0f                ;Add volume
                sta $d418
                rts

mt_effect:      ldy chn_vibindex,x      ;This code used for both pulse
                beq mt_noeffect         ;& vibrato routine
                tya
                and #$01                ;If the table index is odd,
                beq mt_nonewtime        ;get new time
mt_newtime:     lda pulsevibtbl-1,y
                cmp #JUMP               ;Jump command?
                beq mt_effectjump
                sta chn_vibtime,x
                inc chn_vibindex,x
                rts
mt_effectjump:  lda pulsevibtbl,y       ;Jump to position 0 ends
                sta chn_vibindex,x      ;effect
                bne mt_effect
mt_noeffect:    rts
mt_nonewtime:   dec chn_vibtime,x       ;Decrease timecounter
                bne mt_timeleft
                inc chn_vibindex,x      ;If time zero, move table index to
mt_timeleft:    lda pulsevibtbl-1,y     ;next odd position
                tay                     ;(will get new time)
                asl
                sta mt_temp3
                tya
                bmi mt_freqdown

mt_frequp:      lda chn_freqlo,x        ;Subroutines for
                clc                     ;frequency up/down slides
                adc mt_temp3
                sta chn_freqlo,x
                bcc mt_frequpdone
                inc chn_freqhi,x
mt_frequpdone:  rts

mt_freqdown:    lda chn_freqlo,x
                sec
                sbc mt_temp3
                sta chn_freqlo,x
                bcs mt_freqdowndone
                dec chn_freqhi,x
mt_freqdowndone:rts

mt_16bitcompare:lda chn_freqhi,x        ;Subroutine for 16-bit
                cmp mt_temp2            ;frequency compare
                bne mt_16bitdone
mt_checklo:     lda chn_freqlo,x
                cmp mt_temp1
mt_16bitdone:   rts

mt_getnewnote:  lda #$00                ;Assume:no slide
                sta chn_slide,x
                ldy chn_pattnum,x
                lda (mt_patttbllo),y
                sta mt_temp1
                iny
                lda (mt_patttbllo),y
                sta mt_temp2
                ldy chn_pattindex,x
                lda (mt_temp1),y
                sta mt_temp3            ;Store note
                bpl mt_nodur            ;Check 7th bit: Duration command?
                iny
                lda (mt_temp1),y
                sta chn_duration,x      ;Store duration
mt_nodur:       bit mt_temp3            ;Check 6th bit: Slide or Snd command?
                bvc mt_noteok
                iny
                lda (mt_temp1),y
                bmi mt_setslide
mt_setsnd:      sta chn_sound,x
                bpl mt_noteok           ;Sound number always positive
mt_setslide:    asl
                sta chn_slide,x         ;Save slidespeed
mt_noteok:      iny
                lda (mt_temp1),y        ;Check end of pattern
                cmp #$ff
                beq mt_endpatt
                tya
mt_endpatt:     sta chn_pattindex,x
                lda mt_temp3            ;Check note
                and #$3f
                cmp #REST
                beq mt_rest             ;Is it rest or continue?
                bcs mt_cont
                adc chn_transpose,x
                sta chn_note,x
                lda chn_slide,x         ;If slide, don't perform hardrestart
                bne mt_cont
mt_hardrestart: sta $d404,x
                sta $d405,x
                sta $d406,x
                inc chn_initflag,x
                jmp mt_nextchn
mt_rest:        lda chn_waveform,x
                and #$fe
                sta chn_waveform,x
mt_cont:        jmp mt_vibeffect

mt_nextpatt:    lda #$00                ;Reset patternindex
                sta chn_pattindex,x
                tay
                lda chn_songptrlo,x     ;Get songpointer
                sta mt_temp1
                lda chn_songptrhi,x
                sta mt_temp2
mt_getpattern:  lda (mt_temp1),y        ;Get next pattern
                bpl mt_pattok
                iny
                cmp #REPEAT
                bcc mt_settrans
                cmp #JUMP
                beq mt_songjump
                cmp #RET
                bcc mt_setrepeat
                bne mt_songcall
mt_songret:     lda chn_returnlo,x
                sta mt_temp1
                lda chn_returnhi,x
                sta mt_temp2
                ldy #$01
                bne mt_getpattern
mt_songcall:    tya
                adc mt_temp1
                sta chn_returnlo,x
                lda mt_temp2
                adc #$00
                sta chn_returnhi,x
                bne mt_songjump
mt_setrepeat:   sbc #REPEAT             ;Carry=0, subtract one more
                sta chn_repeat,x        ;on purpose
                bne mt_getpattern
mt_settrans:    sbc #TRANS-1            ;Carry=0
                sta chn_transpose,x     ;Set transpose
                jmp mt_getpattern
mt_songjump:    lda (mt_temp1),y        ;Get position where to jump
                pha
                iny
                lda (mt_temp1),y
                sta mt_temp2
                pla
                sta mt_temp1
                ldy #$00
                beq mt_getpattern
mt_pattok:      asl
                sta chn_pattnum,x       ;Save patternnumber
                lda chn_repeat,x        ;Repeats left?
                beq mt_norepeat
                dey
                dec chn_repeat,x
mt_norepeat:    iny
                tya
                clc
                adc mt_temp1
                sta chn_songptrlo,x
                lda mt_temp2
                adc #$00
                sta chn_songptrhi,x     ;Songptr highbyte always nonzero
                jmp mt_vibeffect        ;Do only vibrato/arpeggio to save time

;Ŀ
;Music routine variables and tables                                           
;

chn_counter:    dc.b 0           ;This is the first variable that is zeroed
chn_repeat:     dc.b 0
chn_pattindex:  dc.b 0
chn_vibindex:   dc.b 0
chn_pulseindex: dc.b 0
chn_arpindex:   dc.b 0
chn_slide:      dc.b 0

                dc.b 0,0,0,0,0,0,0
                dc.b 0,0,0,0,0,0,0

chn_waveform:   dc.b 0
chn_songptrlo:  dc.b 0
chn_songptrhi:  dc.b 0
chn_transpose:  dc.b 0
chn_sound:      dc.b 0
chn_duration:   dc.b 0
chn_initflag:   dc.b 0          ;This is the last variable that is zeroed

                dc.b 0,0,0,0,0,0,0
                dc.b 0,0,0,0,0,0,0

chn_note:       dc.b 0
chn_freqlo:     dc.b 0
chn_pulselo:    dc.b 0
chn_freqhi:     dc.b 0
chn_pulsehi:    dc.b 0
chn_vibtime:    dc.b 0
chn_pulsetime:  dc.b 0

                dc.b 0,0,0,0,0,0,0
                dc.b 0,0,0,0,0,0,0

chn_pattnum:    dc.b 0
chn_returnlo:   dc.b 0
chn_returnhi:   dc.b 0
chn_nextchnnum: dc.b 7
chn_unused1:    dc.b 0
chn_unused2:    dc.b 0
chn_unused3:    dc.b 0

                dc.b 0,0,0,14,0,0,0
                dc.b 0,0,0,0

freqtbllo:      dc.b $19,$29,$3b,$4e,$62,$77,$8d,$a5,$be,$d8,$f4,$12
                dc.b $32,$53,$76,$9c,$c4,$ee,$1a,$4a,$7c,$b1,$e9,$24
                dc.b $64,$a6,$ed,$38,$88,$dc,$35,$94,$f8,$62,$d2,$49
                dc.b $c8,$4d,$db,$71,$10,$b8,$6b,$28,$f0,$c4,$a5,$93
                dc.b $90,$9b,$b6,$e2,$20,$71,$d6,$50,$e0,$89,$4a,$27
                dc.b $20,$36,$6d,$c5,$41,$e2,$ac,$a0,$c1,$12,$95,$4e
                dc.b $40,$6d,$da,$8a,$82,$c5,$59,$41,$83,$25,$2b,$9d
                dc.b $80,$da,$b4,$15,$04,$8b,$b2,$83,$07,$4a,$57,$ff

freqtblhi:      dc.b $01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$01,$02
                dc.b $02,$02,$02,$02,$02,$02,$03,$03,$03,$03,$03,$04
                dc.b $04,$04,$04,$05,$05,$05,$06,$06,$06,$07,$07,$08
                dc.b $08,$09,$09,$0a,$0b,$0b,$0c,$0d,$0d,$0e,$0f,$10
                dc.b $11,$12,$13,$14,$16,$17,$18,$1a,$1b,$1d,$1f,$21
                dc.b $23,$25,$27,$29,$2c,$2e,$31,$34,$37,$3b,$3e,$42
                dc.b $46,$4a,$4e,$53,$58,$5d,$63,$69,$6f,$76,$7d,$84
                dc.b $8c,$94,$9d,$a7,$b1,$bb,$c6,$d2,$df,$ec,$fa,$ff

;Ŀ
;Music data!                                                                  
;

SND_TOM         = 0
SND_SLOWARPMIN  = 1
SND_BASS        = 2
SND_SNARE2      = 3
SND_BD2         = 4
SND_SAWLEAD     = 5

snd_ad:         dc.b $08,$04,$07,$0a,$09,$24
snd_sr:         dc.b $79,$5a,$99,$aa,$a6,$ac
snd_pulse:      dc.b $08,$05,$08,$08,$08,$00
snd_pulseindex: dc.b $00,$15,$0b,$00,$00,$00
snd_vibindex:   dc.b $00,$00,$00,$00,$00,$01
snd_arpindex:   dc.b $05,$13,$1d,$23,$2f,$01
snd_filttype:   dc.b $00,$00,$00,$00,$00,$00
snd_filtcutoff: dc.b $00,$00,$00,$00,$00,$00

pulsevibtbl:    dc.b $0c,$00    ;1
                dc.b $02,$24    ;3
                dc.b $03,$b0    ;5
                dc.b $03,$30    ;7
                dc.b JUMP,$05   ;9
                dc.b $03,$c0    ;b
                dc.b $01,$00    ;d
                dc.b $20,$90    ;f
                dc.b $20,$10    ;11
                dc.b JUMP,$0f   ;13
                dc.b $10,$28    ;15
                dc.b $50,$08    ;17
                dc.b $50,$88    ;19
                dc.b JUMP,$17   ;1b

arptbl:         dc.b $21,$00    ;1
                dc.b JUMP,$00   ;3
                dc.b $81,$c4    ;5
                dc.b $41,$14    ;7
                dc.b $40,$12    ;9
                dc.b $0f,$0e    ;b
                dc.b $0d,$0c    ;d
                dc.b $0b,$0a    ;f
                dc.b JUMP,$00   ;11
                dc.b $41,$00    ;13
                dc.b $00        ;15
                dc.b $03        ;16
                dc.b $03        ;17
                dc.b $07        ;18
                dc.b $07        ;19
                dc.b $00        ;1a
                dc.b JUMP,$15   ;1b
                dc.b $81,$c6    ;1d
                dc.b $41,$00    ;1f
                dc.b JUMP,$00   ;21
                dc.b $81,$c8    ;23
                dc.b $41,$ac    ;25
                dc.b $41,$a6    ;27
                dc.b $80,$c8    ;29
                dc.b $80,$c4    ;2b
                dc.b JUMP,$29   ;2d
                dc.b $81,$c4    ;2f
                dc.b $41,$a8    ;31
                dc.b $40,$a4    ;33
                dc.b $40,$a0    ;35
                dc.b $40,$9c    ;37
                dc.b JUMP,$00   ;39

songtbl:        dc.w song0patttbl
                dc.w song0chn1
                dc.w song0chn2
                dc.w song0chn3

;Ŀ
;Loading music - "On The Level Again" by Temuz                                
;

song0patttbl:   dc.w song0patt0
                dc.w song0patt1
                dc.w song0patt2
                dc.w song0patt3
                dc.w song0patt4
                dc.w song0patt5
                dc.w song0patt6
                dc.w song0patt7
                dc.w song0patt8
                dc.w song0patt9
                dc.w song0patt10
                dc.w song0patt11
                dc.w song0patt12
                dc.w song0patt13
                dc.w song0patt14
                dc.w song0patt15
                dc.w song0patt16
                dc.w song0patt17
                dc.w song0patt18
                dc.w song0patt19
                dc.w song0patt20

song0chn1:      dc.b 0,1,0,2
                dc.b 0,1,0,2
                dc.b CALL
                dc.w song0sub1
                dc.b TRANS+1
                dc.b CALL
                dc.w song0sub1
                dc.b TRANS
                dc.b 10
                dc.b JUMP
                dc.w song0chn1

song0chn2:      dc.b REPEAT+3,12
                dc.b 14,19
                dc.b REPEAT+24,13
                dc.b 14,20
                dc.b JUMP
                dc.w song0chn2

song0chn3:      dc.b 11,11
                dc.b 15,16,17,16,18
                dc.b TRANS+1
                dc.b 15,16,17,16,18
                dc.b TRANS+12
                dc.b 10
                dc.b TRANS
                dc.b JUMP
                dc.w song0chn3


song0sub1:      dc.b 3,4,3,5
                dc.b 3,4,3,6
                dc.b 3,4,3,5
                dc.b 7,8,7,9
                dc.b 3,4,3,5
                dc.b 7,8,3,5,RET


song0patt0:     dc.b G2+DUR+CMD,6,SND_BASS
                dc.b D3
                dc.b G2
                dc.b G2
                dc.b B2
                dc.b G2
                dc.b G2
                dc.b C3
                dc.b G2
                dc.b G2
                dc.b A2
                dc.b G2
                dc.b B2
                dc.b G2,ENDPATT

song0patt1:     dc.b D3,CONT,ENDPATT

song0patt2:     dc.b A2,CONT,ENDPATT

song0patt3:     dc.b G2+DUR,6
                dc.b G2
                dc.b B2
                dc.b G2
                dc.b D3
                dc.b G2
                dc.b A2
                dc.b F2
                dc.b G2
                dc.b G2,ENDPATT

song0patt4:     dc.b B2
                dc.b G2
                dc.b D3
                dc.b D3
                dc.b B2
                dc.b B2,ENDPATT

song0patt5:     dc.b B2
                dc.b G2
                dc.b D3
                dc.b D3
                dc.b B2+DUR,3
                dc.b A2+CMD,SLIDE+$7f
                dc.b G2+CMD,SLIDE+$7f
                dc.b F2+CMD,SLIDE+$7f,ENDPATT

song0patt6:     dc.b C3
                dc.b C3
                dc.b G2
                dc.b G2
                dc.b B2
                dc.b B2,ENDPATT

song0patt7:     dc.b F2+DUR,6
                dc.b F2
                dc.b G2
                dc.b F2
                dc.b C3
                dc.b F2
                dc.b G2
                dc.b DIS2
                dc.b F2
                dc.b F2,ENDPATT

song0patt8:     dc.b G2
                dc.b F2
                dc.b C3
                dc.b C3
                dc.b G2
                dc.b G2,ENDPATT

song0patt9:     dc.b B2
                dc.b B2
                dc.b F2
                dc.b F2
                dc.b G2
                dc.b G2,ENDPATT

song0patt10:    dc.b D3+DUR,6
                dc.b C3
                dc.b B2
                dc.b A2
                dc.b G2
                dc.b A2
                dc.b G2
                dc.b F2
                dc.b E2
                dc.b DIS2
                dc.b D2
                dc.b F2
                dc.b G2+DUR,24,ENDPATT

song0patt11:    dc.b REST+DUR,192,ENDPATT

song0patt12:    dc.b G1+DUR+CMD,6,SND_BD2
                dc.b G1+DUR,18
                dc.b G1+DUR+CMD,24,SND_SNARE2
                dc.b G1+DUR+CMD,6,SND_BD2
                dc.b CONT
                dc.b G1
                dc.b G1
                dc.b G1+DUR+CMD,24,SND_SNARE2,ENDPATT

song0patt13:    dc.b G1+DUR+CMD,6,SND_BD2
                dc.b G1
                dc.b G1,CONT
                dc.b G1+CMD,SND_SNARE2,CONT
                dc.b G1+CMD,SND_BD2
                dc.b G1
                dc.b G1
                dc.b G1
                dc.b G1,CONT
                dc.b G1+DUR+CMD,24,SND_SNARE2,ENDPATT

song0patt14:    dc.b C2+DUR+CMD,6,SND_SNARE2
                dc.b C2
                dc.b CONT
                dc.b C2
                dc.b E2+CMD,SND_TOM
                dc.b E2
                dc.b CONT
                dc.b E2
                dc.b C2
                dc.b C2
                dc.b C2
                dc.b C2,ENDPATT

song0patt15:    dc.b G4+DUR+CMD,192,SND_SLOWARPMIN,CONT,ENDPATT

song0patt16:    dc.b D4+DUR+CMD,36,SND_SAWLEAD
                dc.b G3+DUR,6
                dc.b G3+DUR,30
                dc.b D4+DUR,24
                dc.b DIS4+DUR,12
                dc.b D4
                dc.b C4
                dc.b B3
                dc.b A3
                dc.b G3
                dc.b B3
                dc.b D4
                dc.b C4+DUR,36
                dc.b F3+DUR,6
                dc.b F3+DUR,30,ENDPATT

song0patt17:    dc.b C4+DUR,24
                dc.b G3+DUR,6
                dc.b A3
                dc.b B3
                dc.b A3+DUR,12
                dc.b G3
                dc.b F3+DUR,6
                dc.b D3+DUR,24
                dc.b G3,ENDPATT

song0patt18:    dc.b A3+DUR,24
                dc.b G3+DUR,48,REST,ENDPATT

song0patt19:    dc.b A1+DUR,12,A1,ENDPATT

song0patt20:    dc.b G1+DUR+CMD,24,SND_SNARE2,ENDPATT


