;ͻ
;Metal Warrior III by Cadaver: Raster interrupt routines                      
;Started in 5/2000!                                                           
;ͼ

RASTER_GSCREEN  = 0             ;Interrupt at top of screen
RASTER_PANEL    = 1             ;Interrupt at score panel blanking begin
RASTER_PANEL2   = 2             ;Interrupt at score panel blanking end
RASTER_SPRITE   = 3             ;Interrupt for sprite multiplex

RPOS_GSCREEN    = $10
RPOS_PANEL      = $c6
RPOS_PANEL2     = $cd

;Ŀ
;WAITPANEL                                                                    
;                                                                             
;Waits for the score-panel raster interrupt.                                  
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

waitpanel:      lda rastercount
waitpanelloop:  cmp rastercount
                beq waitpanelloop
waitpanel_resty:rts

waitpanel2:     lda $d012
                cmp #RPOS_PANEL2
                bcc waitpanel2
                rts

;Ŀ
;RASTER_GSCREEN, RASTER_SPRITE                                                
;                                                                             
;Raster interrupts for setting up the game screen, and sprite multiplexing.   
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: -                                                                  
;

raster_gscreen: cld
                sta savea
                stx savex
                sty savey
                lda screenmode                  ;Gamescreen or textmode?
                bne rgscr_game
                lda #$18                        ;X-scrolling in place
                sta $d016
textd018:       lda #TEXT_D018                  ;Set textscreen screen ptr.
                sta $d018                       ;and charset
                jsr raster_setpanelcolors
                tay                             ;Use screen 0 for sprites
textscrolly:    lda #$17                        ;Textmode scrolling
                sta $d011
                bne rgscr_sprites

rgscr_game:     ldy screen
                lda gscrd018tbl,y               ;Correct screendata ptr
                sta $d018
                lda loadedlevel                 ;In the fortress?
                cmp #12
                bne rgscr_normalcolor
                lda plotbits                    ;This is totally wrong, but
                and #$02                        ;fast :-) (should use the
                tax                             ;plotbit subroutines)
                bpl rgscr_colorkludge
rgscr_normalcolor:
                ldx colormode
rgscr_colorkludge:
                lda bgcols,x                    ;Set background colors
                sta $d021
                lda bgcols+4,x
                sta $d022
                lda bgcols+8,x
                sta $d023
gamescrollx:    lda #$14
                sta $d016
gamescrolly:    lda #$14
                sta $d011

rgscr_sprites:  lda frameptrtbl,y               ;Frames must go to the
                sta rspr_load7f+2               ;correct screen
                sta rspr_load6f+2
                sta rspr_load5f+2
                sta rspr_load4f+2
                sta rspr_load3f+2
                sta rspr_load2f+2
                sta rspr_load1f+2
                sta rspr_load0f+2
                lda newsprflag                  ;New sprites to display?
                beq rgscr_nonewspr
                dec newsprflag
                lda sortsprstart
                sta irqsprstart
                lda sortirqstart
                sta irqirqstart
                lda sortirqend
                sta irqirqend
                lda sortsprend
                sec
                sbc sortsprstart
                tax
                cpx #$09                        ;More than 8 sprites?
                bcc rgscr_sprnotover
                ldx #$08
rgscr_sprnotover:
                lda sprontbl,x
                sta rgscr_nonewspr+1
rgscr_nonewspr: lda #$00
                sta $d015
                bne rgscr_notzerospr            ;Are there sprites to be done?
                ldx #RASTER_PANEL
                jmp endraster
rgscr_notzerospr:
                ldx irqirqstart
                stx irqirqcount
                lda sprirqend,x
                sta irqcmp
                jmp rspr_direct                 ;First spriteIRQ happens right
                                                ;now

rspr_load7:     lda sortspry,y
                sta $d00f
                lda sortsprxl,y
                sta $d00e
                lda $d010
                and #$7f
                ora sortsprxh,y
                sta $d010
                ldx sortsprf,y
rspr_load7f:    stx screen1+$3ff
                lda sprcoltbl-$40,x
                sta $d02e
                iny
                cpy irqcmp
                bcc rspr_load6
                jmp rspr_loadend
rspr_load6:     lda sortspry,y
                sta $d00d
                lda sortsprxl,y
                sta $d00c
                lda $d010
                and #$bf
                ora sortsprxh,y
                sta $d010
                ldx sortsprf,y
rspr_load6f:    stx screen1+$3fe
                lda sprcoltbl-$40,x
                sta $d02d
                iny
                cpy irqcmp
                bcc rspr_load5
                jmp rspr_loadend
rspr_load5:     lda sortspry,y
                sta $d00b
                lda sortsprxl,y
                sta $d00a
                lda $d010
                and #$df
                ora sortsprxh,y
                sta $d010
                ldx sortsprf,y
rspr_load5f:    stx screen1+$3fd
                lda sprcoltbl-$40,x
                sta $d02c
                iny
                cpy irqcmp
                bcc rspr_load4
                jmp rspr_loadend
rspr_load4:     lda sortspry,y
                sta $d009
                lda sortsprxl,y
                sta $d008
                lda $d010
                and #$ef
                ora sortsprxh,y
                sta $d010
                ldx sortsprf,y
rspr_load4f:    stx screen1+$3fc
                lda sprcoltbl-$40,x
                sta $d02b
                iny
                cpy irqcmp
                bcc rspr_load3
                jmp rspr_loadend
rspr_load3:     lda sortspry,y
                sta $d007
                lda sortsprxl,y
                sta $d006
                lda $d010
                and #$f7
                ora sortsprxh,y
                sta $d010
                ldx sortsprf,y
rspr_load3f:    stx screen1+$3fb
                lda sprcoltbl-$40,x
                sta $d02a
                iny
                cpy irqcmp
                bcc rspr_load2
                jmp rspr_loadend
rspr_load2:     lda sortspry,y
                sta $d005
                lda sortsprxl,y
                sta $d004
                lda $d010
                and #$fb
                ora sortsprxh,y
                sta $d010
                ldx sortsprf,y
rspr_load2f:    stx screen1+$3fa
                lda sprcoltbl-$40,x
                sta $d029
                iny
                cpy irqcmp
                bcc rspr_load1
                jmp rspr_loadend
rspr_load1:     lda sortspry,y
                sta $d003
                lda sortsprxl,y
                sta $d002
                lda $d010
                and #$fd
                ora sortsprxh,y
                sta $d010
                ldx sortsprf,y
rspr_load1f:    stx screen1+$3f9
                lda sprcoltbl-$40,x
                sta $d028
                iny
                cpy irqcmp
                bcc rspr_load0
                jmp rspr_loadend
rspr_load0:     lda sortspry,y
                sta $d001
                lda sortsprxl,y
                sta $d000
                lda $d010
                and #$fe
                ora sortsprxh,y
                sta $d010
                ldx sortsprf,y
rspr_load0f:    stx screen1+$3f8
                lda sprcoltbl-$40,x
                sta $d027
                iny
                cpy irqcmp
                bcs rspr_loadend
                jmp rspr_load7

rspr_loadend:   ldx irqirqcount
                inx
                cpx irqirqend
                bcs rspr_alldone
                stx irqirqcount
                lda sprirqend,x
                sta irqcmp
                sbc sprirqstart,x               ;Carry is clear, subtract one
                tay                             ;more on purpose
                lda sprirqline,x
                sbc sprlinesubtbl,y             ;Carry is now set
                sta rasterpos+RASTER_SPRITE
                clc                             ;Subtract one more on purpose
                sbc $d012                       ;Late?
                bcc rspr_direct
                beq rspr_direct
                ldx #RASTER_SPRITE

;Ŀ
;ENDRASTER                                                                    
;                                                                             
;Common End-Of-Interrupt part for all raster interrupts.                      
;                                                                             
;Parameters: X:Number of next raster IRQ                                      
;Returns: -                                                                   
;Modifies: A                                                                  
;

endraster:      dec $d019                       ;Acknowledge IRQ
                lda rasterroutlo,x              ;Set vector & raster position
                sta $fffe                       ;for next IRQ
                lda rasterrouthi,x
                sta $ffff
                lda rasterpos,x
                sta $d012
                lda savea
                ldx savex
                ldy savey
nmi:            rti

rspr_alldone:   lda #RPOS_PANEL-2               ;Late from the panelirq?
                cmp $d012
                bcc rspr_latepanel
                ldx #RASTER_PANEL
                jmp endraster
rspr_latepanel: lda #$57                        ;Blank screen
                sta $d011
                bne rpanel_direct

raster_sprite:  cld
                sta savea
                stx savex
                sty savey
                ldx irqirqcount
rspr_direct:    lda sprirqstart,x
                tay
                and #$07
                tax
                lda rspr_jumptbllo,x
                sta rspr_jump+1
                lda rspr_jumptblhi,x
                sta rspr_jump+2
rspr_jump:      jmp $1000

;Ŀ
;RASTER_PANEL, RASTER_PANEL2                                                  
;                                                                             
;Raster interrupt for setting up the score panel.                             
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: -                                                                  
;

raster_panel:   cld
                sta savea
                lda $d011
                cmp #$16
                beq rpanel_badline
                lda #$17
                sta $d011
                pha
                pla
                pha
                pla
                nop
                nop
                nop
                nop
rpanel_badline: lda #$57                        ;Blank screen
                sta $d011
                stx savex
                sty savey
rpanel_direct:  lda #TEXT_D018                  ;Set textscreen screen ptr.
                sta $d018                       ;and charset
                lda #EMPTYSPRITE                ;Set frame of all sprites
                sta textscreen+$3ff             ;to empty
                sta textscreen+$3fe
                sta textscreen+$3fd
                sta textscreen+$3fc
                sta textscreen+$3fb
                sta textscreen+$3fa
                sta textscreen+$3f9
                sta textscreen+$3f8
                lda #$00                        ;All sprites off
                sta $d015
                inc rastercount
                ldx #RASTER_PANEL2
                lda #$18                        ;X-scrolling in place
                sta $d016
                jmp endraster

raster_panel2:  cld
                sta savea
                stx savex
                sty savey
                jsr raster_setpanelcolors
raster_panel2wait:
                lda $d012
                cmp #RPOS_PANEL2+2
                bcc raster_panel2wait
                lda #$17                        ;Screen on
                sta $d011
                jsr music                       ;Play music
                ldx #RASTER_GSCREEN             ;Start from gamescreen again
                jmp endraster

idleirq:        lda fastloadstatus              ;In fakeIRQload-mode, redirect IRQs
                bmi redirectirq
                dec $d019
                jmp $ea81

redirectirq:    ldx $01
                lda #$35                        ;Note: this will necessarily have overhead,
                sta $01                         ;which means that the sensitive IRQs like
                lda #>rr_return                 ;the panel-split should take extra advance
                pha
                lda #<rr_return
                pha
                php
                jmp ($fffe)
rr_return:      stx $01
                jmp $ea81

raster_setpanelcolors:
                lda #$00
                ldx #$0b                        ;Set panel multicolors
                ldy #$0c
                sta $d021
                stx $d022
                sty $d023
                rts
