;ͻ
;Metal Warrior III by Cadaver: Screen+scrolling routines                      
;Started in 5/2000!                                                           
;ͼ

;Ŀ
;CLEARTEXTSCREEN                                                              
;                                                                             
;Clears the upmost 19 rows of the text screen. Sets screenmode to textscreen! 
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

cleartextscreen:jsr waitpanel
                jsr clearsprites
cleartextscreen2:
                lda #$ff                        ;Set all sprites multicolor
                jsr setsprmulticolor
                lda #$17                        ;Reset textmode scrolling
                ldx #$20
                ldy #TEXT_D018
ctscommon:      sty textd018+1
                sta textscrolly+1
                stx ctsloop2+1
                ldx #$00
                stx screenmode                  ;Mode = textscreen
ctsloop1:       lda #$00
                jsr setrowcolor
                inx
                cpx #SCROLLROWS
                bcc ctsloop1
                tax
ctsloop2:       lda #$20
                jsr setrowchar
                inx
                cpx #SCROLLROWS
                bcc ctsloop2
                rts

setsprmulticolor:
                sta $d01c                       ;Sprite multicolor mode
                eor #$ff
                sta $d01d                       ;Sprite X-expand
                rts

;Ŀ
;SETROWCHAR                                                                   
;                                                                             
;Sets all characters on a textscreen row                                      
;                                                                             
;Parameters: X:Row number A:Char number                                       
;Returns: -                                                                   
;Modifies: Y                                                                  
;

setrowchar:     tay
                lda #>textscreen
setrowcommon:   ora screenrowtblhi,x
                sta setrowcharloop+2
                lda screenrowtbllo,x
                sta setrowcharloop+1
                tya
                ldy #39
setrowcharloop: sta $1000,y
                dey
                bpl setrowcharloop
                rts

;Ŀ
;SETROWCOLOR                                                                  
;                                                                             
;Sets color of all characters on a textscreen row.                            
;                                                                             
;Parameters: X:Row number A:Char number                                       
;Returns: -                                                                   
;Modifies: Y                                                                  
;

setrowcolor:    tay
                lda #>colors
                bne setrowcommon


;Ŀ
;SCROLLTEXT                                                                   
;                                                                             
;Scrolls textscreen smoothly (actually redraws whole screen from (textptr))   
;                                                                             
;Parameters: A:Speed                                                          
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

st_display:     lda #$00
                tax
                beq st_common
scrolltext:     sta st_sub+1
                lda textscrolly+1
                sec
st_sub:         sbc #$00
                cmp #$10
                bcc st_scrolldown
                cmp #$18
                bcs st_scrollup
st_normalize:   and #$07
                ora #$10
                sta textscrolly+1
                rts
st_scrolldown:  jsr st_normalize
                lda #<40
                ldx #>40
                bpl st_common
st_scrollup:    jsr st_normalize
                lda #<(-40)
                ldx #>(-40)
st_common:      clc
                adc textptrlo
                sta textptrlo
                txa
                adc textptrhi
                sta textptrhi
                ldx #$00
st_rowloop:     lda screenrowtbllo,x
                sta st_sta+1
                clc
                adc textptrlo
                sta st_lda+1
                lda screenrowtblhi,x
                pha
                ora #>textscreen
                sta st_sta+2
                pla
                adc textptrhi
                sta st_lda+2
                ldy #39
st_lda:         lda $1000,y
st_sta:         sta $1000,y
                dey
                bpl st_lda
                inx
                cpx #SCROLLROWS
                bcc st_rowloop
                rts

;Ŀ
;PRINTTEXTC                                                                   
;                                                                             
;Centered text printing                                                       
;                                                                             
;Parameters: Y:row textptr:address of text                                    
;Returns: -                                                                   
;Modifies: A,X,textptr,temp1-temp2,temp6                                      
;

printtextc:     sty temp6
                ldy #$00
                ldx #40
ptc_getlen:     lda (textptrlo),y
                beq ptc_ready
                dex
                iny
                bne ptc_getlen
ptc_ready:      txa
                lsr
                tax
                ldy temp6

;Ŀ
;PRINTTEXT                                                                    
;                                                                             
;Prints a zero-terminated string                                              
;                                                                             
;Parameters: X,Y, textptr:Address of text                                     
;Returns: -                                                                   
;Modifies: A,temppptr,temp1-temp2,temp6                                       
;

printtext:      sty temp6
                txa
                clc
                adc screenrowtbllo,y
                sta temp1
                lda #>textscreen
                adc screenrowtblhi,y
                sta temp2
                ldy #$00
pt_printloop:   lda (textptrlo),y
                beq pt_printend
                sta (temp1),y
                iny
                jmp pt_printloop
pt_printend:    tya
                sec                             ;Add one more; INY not needed
                adc textptrlo
                sta textptrlo
                bcc pt_notover3
                inc textptrhi
pt_notover3:    ldy temp6
                rts

;Ŀ
;INITBLOCKS                                                                   
;                                                                             
;Calculates the address for every block & map-row. (We hate multiplication!)  
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,temp1-temp4                                                    
;

initblocks:     ldx #$00
                lda #<mapdata
                sta temp1
                lda #>mapdata
                sta temp2
                lda #<blocks
                sta temp3
                lda #>blocks
                sta temp4
ib_loop:        lda temp1
                sta maptbllo,x
                clc
                adc mapsizex
                sta temp1
                lda temp2
                sta maptblhi,x
                adc #$00
                sta temp2
ib_loop2:       lda temp3
                sta blktbllo,x
                clc
                adc #$10
                sta temp3
                lda temp4
                sta blktblhi,x
                adc #$00
                sta temp4
                inx
                bpl ib_loop
                cpx #$c0
                bcc ib_loop2
                rts

;Ŀ
;INITSCROLL                                                                   
;                                                                             
;Resets & stops scrolling.                                                    
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

initscroll:     lda #$00
                sta scrollsx
                sta scrollsy
                sta scrollcsx
                sta scrollcsy
                sta scrcounter
                sta scradd
                lda #$04
                sta scrollx
                sta scrolly
                ora #$10
                sta gamescrollx+1
                sta gamescrolly+1
                jmp sl_calcspritesub            ;Calculate also spritesubtract-
                                                ;value

;Ŀ
;SCROLLLOGIC                                                                  
;                                                                             
;Performs the "logic" of the scrolling, can be called any time.               
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

scrolllogic:    lda scradd                      ;If speed is zero, look out
                beq sl_getnewspeed              ;for a new speed-setting
                lda scrollx                     ;Update finescroll-counters
                sec
                sbc scrollcsx                   ;(don't need to care of
                and #$07                        ;overflow right now)
                sta scrollx
                lda scrolly
                sec
                sbc scrollcsy
                and #$07
                sta scrolly
                lda scrcounter                  ;Then increase the so-called
                clc                             ;workcounter which tells what
                adc scradd                      ;to do at this point
                sta scrcounter
                cmp #$08                        ;If it's >7 then this scrolling
                bcs sl_getnewspeed              ;is ready
                cmp #$04                        ;When it's 4, swap the
                bne sl_dontswap                 ;"new" block & map coords
                lda newblockx
                sta blockx
                lda newblocky
                sta blocky
                lda newmapx
                sta mapx
                lda newmapy
                sta mapy
                lda loadedlevel                 ;Flight level kludge?
                cmp #LEVEL_FLIGHT
                bcc sl_noflight
                lda mapx
                adc #$09
                cmp mapsizex
                bcc sl_noflight
                inc roundtrips
                lda #$00
                sta mapx
sl_noflight:    lda #$00                        ;Clear the speed request
                sta scrollsx                    ;at this point
                sta scrollsy
sl_dontswap:    jmp sl_calcspritesub
sl_getnewspeed: lda #$00                        ;Reset the workcounter
                sta scrcounter
                jsr sl_copytonew
                lda #$05                        ;Reset shift direction (center)
                sta shiftdir
                lda scrollsx                    ;Get the "requested" speed
                sta scrollcsx
                beq sl_nox2
                bmi sl_xneg2
sl_xpos2:       lda mapx                        ;Are we on the edge of map?
                clc                             ;(right)
                adc #$0a
                cmp mapsizex
                bcc sl_xpos2ok
                lda blockx
                cmp #$01
                bcc sl_xpos2ok
sl_xzero:       lda #$00
                sta scrollcsx
                beq sl_nox2
sl_xpos2ok:     inc shiftdir
                inc newblockx                   ;Change block & mappointers
                lda newblockx
                cmp #$04
                bcc sl_nomapr
                lda #$00
                sta newblockx
                inc newmapx
sl_nomapr:      lda #$03
                sta scrollx
                bne sl_nox2
sl_xneg2:       lda blockx                      ;Are we on the edge of map?
                bne sl_xneg2ok                  ;(left)
                lda mapx
                bne sl_xneg2ok
                beq sl_xzero
sl_xneg2ok:     dec shiftdir
                dec newblockx                   ;Change block & mappointers
                bpl sl_nomapl
                lda #$03
                sta newblockx
                dec newmapx
sl_nomapl:      lda #$04
                sta scrollx
sl_nox2:        lda scrollsy
                sta scrollcsy
                beq sl_noy2
                bmi sl_yneg2
sl_ypos2:       lda mapy                        ;Are we on the edge of map?
                clc                             ;(bottom)
                adc #$05
                cmp mapsizey
                bcc sl_ypos2ok
sl_yzero:       lda #$00
                sta scrollcsy
                beq sl_noy2
sl_ypos2ok:     lda shiftdir
                clc
                adc #$04
                sta shiftdir
                inc newblocky                   ;Change block & mappointers
                lda newblocky
                cmp #$04
                bcc sl_nomapd
                lda #$00
                sta newblocky
                inc newmapy
sl_nomapd:      lda #$03
                sta scrolly
                bne sl_noy2
sl_yneg2:       lda blocky                      ;Are we on the edge of map?
                bne sl_yneg2ok                  ;(top)
                lda mapy
                bne sl_yneg2ok
                beq sl_yzero
sl_yneg2ok:     lda shiftdir
                sec
                sbc #$04
                sta shiftdir
                dec newblocky                   ;Change block & mappointers
                bpl sl_nomapu
                lda #$03
                sta newblocky
                dec newmapy
sl_nomapu:      lda #$04
                sta scrolly
sl_noy2:        lda scrollcsx
                bpl sl_xnotneg                  ;Take absolute value of X-
                eor #$ff                        ;speed
                clc
                adc #$01
sl_xnotneg:     sta scradd                      ;And save as the workcounter
                sta temp1                       ;addvalue
                lda scrollcsy
                bpl sl_ynotneg
                eor #$ff
                clc
                adc #$01
sl_ynotneg:     sta temp2
                cmp scradd                      ;Is absolute Y-speed greater
                bcc sl_ynothigher               ;than X-speed?
                sta scradd                      ;Then use it as addvalue
sl_ynothigher:  lda scradd                      ;Do we have scrolling at all?
                beq sl_calcspritesub
sl_not0:        cmp #$02                        ;Is the speed 2?
                bne sl_not2                     ;Then speed on both axes must
                lda temp1                       ;be either 0 or 2
                cmp #$01
                bne sl_xnot1
                asl scrollcsx
sl_xnot1:       lda temp2
                cmp #$01
                bne sl_ynot1
                asl scrollcsy
sl_not2:
sl_ynot1:       ldx screen                      ;Which screen?
                lda sw_shifttbllo,x             ;Take the corresponding
                sta sw_shiftjump+1              ;screen shift routine
                sta sw_shiftjump2+1
                lda sw_shifttblhi,x
                sta sw_shiftjump+2
                sta sw_shiftjump2+2
                ldx shiftdir                    ;Which direction?
                lda sw_shcoltbllo,x             ;Take the corresponding
                sta sw_shcoljump+1              ;color shift routine
                lda sw_shcoltblhi,x
                sta sw_shcoljump+2
sl_calcspritesub:
                lda levelnum
                cmp #LEVEL_FLIGHT
                bne sl_cssnormal
                lda #$00
                sta charsubxl
                sta sprsubxh
                lda #$01
                sta charsubxh
                lda #$08
                sta sprsubxl
                bne sl_cssynormal
sl_cssnormal:   lda scrollx
                eor #$07
                asl
                asl
                asl
                sta temp1
                lda blockx
                ror
                ror
                ror
                and #$c0
                sta charsubxl
                ora temp1
                sec
                sbc #248
                sta sprsubxl
                lda mapx
                sbc #0
                sta sprsubxh
                lda mapx
                sta charsubxh
sl_cssynormal:  lda scrolly
                eor #$07
                asl
                asl
                asl
                sta temp1
                lda blocky
                ror
                ror
                ror
                and #$c0
                sta charsubyl
                ora temp1
                sec
                sbc #176
                sta sprsubyl
                lda mapy
                sbc #1
                sta sprsubyh
                lda mapy
                sta charsubyh
                rts
sl_copytonew:   lda blockx
                sta newblockx
                lda blocky
                sta newblocky
                lda mapx
                sta newmapx
                lda mapy
                sta newmapy
                rts

;Ŀ
;SCROLLWORK                                                                   
;                                                                             
;Performs the hard part of scrolling. To be called during the drawing of the  
;gamescreen.                                                                  
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y,temp1-temp5                                                  
;

scrollwork:     lda scrollx
                ora #$10
                sta gamescrollx+1
                lda scrolly
                ora #$10
                sta gamescrolly+1
                lda scradd                      ;If speed 0 do nothing
                beq sw_done
                lda scrcounter
                beq sw_shiftscreen
                cmp #$02
                beq sw_drawblocks
                cmp #$04
                beq sw_shiftcolors
sw_donothing:
sw_done:        rts

sw_shiftscreen: ldy shiftdir
                ldx sw_shiftpostbl,y
                ldy #0                          ;Shift the screen data
                lda #24
sw_shiftjump:   jmp $0000                       ;(on the non-visible screen)

sw_drawblocks:
sw_shiftrestx:  ldx #$00
sw_shiftresty:  ldy #$00
                lda #39
sw_shiftjump2:  jsr $0000                       ;(on the non-visible screen)
                lda scrollcsy                   ;Draw new blocks where
                beq sw_dbnoy                    ;necessary
                bmi sw_dbup
sw_dbdown:      jsr sw_drawdown
                jmp sw_dbnoy
sw_dbup:        jsr sw_drawup
sw_dbnoy:       lda scrollcsx
                beq sw_dbnox
                bmi sw_dbleft
sw_dbright:     jmp sw_drawright
sw_dbleft:      jmp sw_drawleft
sw_dbnox:       rts

sw_shiftcolors: lda screen                      ;Switch the displayed screen,
                eor #$01                        ;and scroll colorscreen
                sta screen
sw_shcoljump:   jmp $0000

sw_shift1:      sta sw_shift1cmp+1
sw_shift1loop:                                  ;Screen shift routine:
N               set 0                           ;From screen1 to screen2
                repeat SCROLLROWS
                lda screen1-40+N*40,x
                sta screen2+N*40,y
N               set N+1
                repend
                inx
                iny
sw_shift1cmp:   cpy #$00
                bne sw_shift1loop
                stx sw_shiftrestx+1
                sty sw_shiftresty+1
                rts

sw_shift2:      sta sw_shift2cmp+1
sw_shift2loop:                                  ;Screen shift routine:
N               set 0                           ;From screen2 to screen1
                repeat SCROLLROWS
                lda screen2-40+N*40,x
                sta screen1+N*40,y
N               set N+1
                repend
                inx
                iny
sw_shift2cmp:   cpy #$00
                bne sw_shift2loop
                stx sw_shiftrestx+1
                sty sw_shiftresty+1
                rts

sw_shcolup:     ldx #38                         ;Color shift routines:
                ldy #38                         ;selfmodifying code :-)
                lda #$ca                ;DEX    ;(but only 3 different
                sta sw_shcolup2topinx           ;shift routines needed for
                sta sw_shcolup2bottominx
                lda #$88                ;DEY    ;all 8 directions)
                sta sw_shcolup2topiny
                sta sw_shcolup2bottominy
                lda #$ff
                jsr sw_shcolup2top
                ldx #38
                ldy #38
                lda #$ff
                jmp sw_shcolup2bottom

sw_shcolupleft: ldx #37
                ldy #38
                lda #$ca                ;DEX
                sta sw_shcolup2topinx
                sta sw_shcolup2bottominx
                lda #$88                ;DEY
                sta sw_shcolup2topiny
                sta sw_shcolup2bottominy
                lda #$ff
                jsr sw_shcolup2top
                jsr sw_drawcollefttop
                ldx #37
                ldy #38
                lda #$ff
                jsr sw_shcolup2bottom
                jmp sw_drawcolleftbottom


sw_shcolupright:ldx #1
                ldy #0
                lda #$e8                ;INX
                sta sw_shcolup2topinx
                sta sw_shcolup2bottominx
                lda #$c8                ;INY
                sta sw_shcolup2topiny
                sta sw_shcolup2bottominy
                lda #39
                jsr sw_shcolup2top
                jsr sw_drawcolrighttop
                ldx #1
                ldy #0
                lda #39
                jsr sw_shcolup2bottom
                jmp sw_drawcolrightbottom

sw_shcolright:  ldx #1
                ldy #0
                lda #$e8                ;INX
                sta sw_shcolright2topinx
                sta sw_shcolright2bottominx
                lda #$c8                ;INY
                sta sw_shcolright2topiny
                sta sw_shcolright2bottominy
                lda #39
                jsr sw_shcolright2top
                jsr sw_drawcolrighttop
                ldx #1
                ldy #0
                lda #39
                jsr sw_shcolright2bottom
                jmp sw_drawcolrightbottom

sw_shcolleft:   ldx #37
                ldy #38
                lda #$ca                ;DEX
                sta sw_shcolright2topinx
                sta sw_shcolright2bottominx
                lda #$88                ;DEY
                sta sw_shcolright2topiny
                sta sw_shcolright2bottominy
                lda #$ff
                jsr sw_shcolright2top
                jsr sw_drawcollefttop
                ldx #37
                ldy #38
                lda #$ff
                jsr sw_shcolright2bottom
                jmp sw_drawcolleftbottom

sw_shcoldown:   ldx #38
                ldy #38
                lda #$ca                ;DEX
                sta sw_shcoldown2topinx
                sta sw_shcoldown2bottominx
                lda #$88                ;DEY
                sta sw_shcoldown2topiny
                sta sw_shcoldown2bottominy
                lda #$ff
                jsr sw_shcoldown2top
                ldx #38
                ldy #38
                lda #$ff
                jmp sw_shcoldown2bottom

sw_shcoldownleft:ldx #37
                ldy #38
                lda #$ca                ;DEX
                sta sw_shcoldown2topinx
                sta sw_shcoldown2bottominx
                lda #$88                ;DEY
                sta sw_shcoldown2topiny
                sta sw_shcoldown2bottominy
                lda #$ff
                jsr sw_shcoldown2top
                jsr sw_drawcollefttop
                ldx #37
                ldy #38
                lda #$ff
                jsr sw_shcoldown2bottom
                jmp sw_drawcolleftbottom

sw_shcoldownright:ldx #1
                ldy #0
                lda #$e8                ;INX
                sta sw_shcoldown2topinx
                sta sw_shcoldown2bottominx
                lda #$c8                ;INY
                sta sw_shcoldown2topiny
                sta sw_shcoldown2bottominy
                lda #39
                jsr sw_shcoldown2top
                jsr sw_drawcolrighttop
                ldx #1
                ldy #0
                lda #39
                jsr sw_shcoldown2bottom
                jmp sw_drawcolrightbottom

sw_shcolup2top: sta sw_shcolup2topcpx+1
sw_shcolup2toploop:
N               set SCROLLSPLIT
                lda colors+N*40,x
                sta vcolstore,x
N               set SCROLLSPLIT-1
                repeat SCROLLSPLIT
                lda colors+N*40,x
                sta colors+40+N*40,y
N               set N-1
                repend
                lda vcolbuf,y
                sta colors,y
sw_shcolup2topiny: iny
sw_shcolup2topinx: inx
sw_shcolup2topcpx: cpx #$00
                bne sw_shcolup2toploop
                rts

sw_shcolup2bottom: sta sw_shcolup2bottomcpx+1
sw_shcolup2bottomloop:
N               set SCROLLROWS-2
                repeat SCROLLROWS-SCROLLSPLIT-2
                lda colors+N*40,x
                sta colors+40+N*40,y
N               set N-1
                repend
                lda vcolstore,x
                sta colors+40+N*40,y
sw_shcolup2bottominy: iny
sw_shcolup2bottominx: inx
sw_shcolup2bottomcpx: cpx #$00
                bne sw_shcolup2bottomloop
                rts

sw_shcolright2top: sta sw_shcolright2topcpx+1
sw_shcolright2toploop:
N               set 0
                repeat SCROLLSPLIT
                lda colors+N*40,x
                sta colors+N*40,y
N               set N+1
                repend
sw_shcolright2topiny:iny
sw_shcolright2topinx:inx
sw_shcolright2topcpx:cpx #$00
                bne sw_shcolright2toploop
                rts

sw_shcolright2bottom: sta sw_shcolright2bottomcpx+1
sw_shcolright2bottomloop:
N               set SCROLLSPLIT
                repeat SCROLLROWS-SCROLLSPLIT
                lda colors+N*40,x
                sta colors+N*40,y
N               set N+1
                repend
sw_shcolright2bottominy:iny
sw_shcolright2bottominx:inx
sw_shcolright2bottomcpx:cpx #$00
                bne sw_shcolright2bottomloop
                rts

sw_shcoldown2top:sta sw_shcoldown2topcpx+1
sw_shcoldown2toploop:
N               set 0
                repeat SCROLLSPLIT
                lda colors+40+N*40,x
                sta colors+N*40,y
N               set N+1
                repend
sw_shcoldown2topiny:iny
sw_shcoldown2topinx:inx
sw_shcoldown2topcpx:cpx #$00
                bne sw_shcoldown2toploop
                rts

sw_shcoldown2bottom:sta sw_shcoldown2bottomcpx+1
sw_shcoldown2bottomloop:
N               set SCROLLSPLIT
                repeat SCROLLROWS-SCROLLSPLIT-1
                lda colors+40+N*40,x
                sta colors+N*40,y
N               set N+1
                repend
                lda vcolbuf,y
                sta colors+SCROLLROWS*40-40,y
sw_shcoldown2bottominy:iny
sw_shcoldown2bottominx:inx
sw_shcoldown2bottomcpx:cpx #$00
                bne sw_shcoldown2bottomloop
                rts



sw_drawcollefttop:ldx #$00                      ;Routines to draw the new
                beq sw_drawcolhtop              ;colors to the edge of the
sw_drawcolrighttop:ldx #38                      ;colorscreen (these use a
sw_drawcolhtop:                                 ;buffer which has been loaded
N               set 0                           ;beforehand; when drawing the
                repeat SCROLLSPLIT              ;new blocks to the screen)
                lda hcolbuf+N
                sta colors+N*40,x
N               set N+1
                repend
                rts

sw_drawcolleftbottom:ldx #$00                   ;Routines to draw the new
                beq sw_drawcolhbottom           ;colors to the edge of the
sw_drawcolrightbottom:ldx #38                   ;colorscreen (these use a
sw_drawcolhbottom:                              ;buffer which has been loaded
N               set SCROLLSPLIT                 ;beforehand; when drawing the
                repeat SCROLLROWS-SCROLLSPLIT   ;new blocks to the screen)
                lda hcolbuf+N
                sta colors+N*40,x
N               set N+1
                repend
                rts

sw_drawleft:    ldx newmapy                        ;Draw new blocks to the left
                lda maptbllo,x
                sta temp3
                lda maptblhi,x
                sta temp4
                lda newmapx
                clc
                adc temp3
                sta temp3
                bcc sw_dlcalcdone
                inc temp4
sw_dlcalcdone:  lda newblockx
                sta temp1
                lda screen
                eor #$01
                tax
                lda scradrtblhi,x
                sta swdl_sta+2
                ldx #$00
swdl_common:    lda #SCROLLROWS-1
                sta temp5
                lda #<hcolbuf
                sta swdl_sta2+1
                lda newblocky
                asl
                asl
                ora temp1
swdl_getblock:  sta temp2
                ldy #$00
                lda (temp3),y
                tay
                lda blktbllo,y
                sta swdl_lda+1
                lda blktblhi,y
                sta swdl_lda+2
                ldy temp2
swdl_lda:       lda $1000,y
swdl_sta:       sta $1000,x
                sta swdl_lda2+1
swdl_lda2:      lda charcols
swdl_sta2:      sta hcolbuf
                inc swdl_sta2+1
                dec temp5
                bmi swdl_ready
                txa
                clc
                adc #40
                tax
                bcc swdl_not2
                inc swdl_sta+2
                clc
swdl_not2:      lda cpd_tbl,y
                tay
                bpl swdl_lda
swdl_block:     lda temp3
                clc
                adc mapsizex
                sta temp3
                bcc swdl_not3
                inc temp4
swdl_not3:      lda temp1
                jmp swdl_getblock
swdl_ready:     rts

sw_drawright:   ldx newmapy                     ;Draw new blocks to the right
                lda maptbllo,x
                sta temp3
                lda maptblhi,x
                sta temp4
                lda newmapx
                clc
                adc #$09
                adc temp3
                sta temp3
                bcc sw_drcalcdone
                inc temp4
sw_drcalcdone:  lda newblockx
                clc
                adc #$02
                pha
                cmp #$04
                bcc swdr_not1
                inc temp3
                bne swdr_not1
                inc temp4
swdr_not1:      pla
                and #$03
                sta temp1
                lda screen
                eor #$01
                tax
                lda scradrtblhi,x
                sta swdl_sta+2
                ldx #38
                jmp swdl_common

sw_drawup:      ldx newmapy                     ;Draw new blocks to top of
                lda maptbllo,x                  ;screen
                sta temp3
                lda maptblhi,x
                sta temp4
                lda newblocky
                asl
                asl
                ora newblockx
                sta temp2
                lda screen
                eor #$01
                tax
                lda #$00
                sta swdu_sta+1
                lda scradrtblhi,x
                sta swdu_sta+2
swdu_common:    lda #<vcolbuf
                sta swdu_sta2+1
                lda #>vcolbuf
                sta swdu_sta2+2
swdu_common2:   ldx #$00
                ldy newmapx
swdu_getblock:  lda (temp3),y
                iny
                sty temp5
                tay
                lda blktbllo,y
                sta swdu_lda+1
                lda blktblhi,y
                sta swdu_lda+2
                ldy temp2
swdu_lda:       lda $1000,y
swdu_sta:       sta screen1,x
                sta swdu_lda2+1
swdu_lda2:      lda charcols
swdu_sta2:      sta vcolbuf,x
                inx
                cpx #39
                bcs swdu_ready
                lda cpr_tbl,y
                tay
                bpl swdu_lda
                and #$0f
                sta temp2
                ldy temp5
                jmp swdu_getblock
swdu_ready:     rts

sw_drawdown:    lda newmapy                     ;Draw new blocks to bottom of
                clc                             ;screen
                adc #$04
                tax
                lda newblocky
                adc #$02
                cmp #$04
                bcc sw_ddcalcdone
                and #$03
                inx
sw_ddcalcdone:  asl
                asl
                ora newblockx
                sta temp2
                lda maptbllo,x
                sta temp3
                lda maptblhi,x
                sta temp4
                lda screen
                eor #$01
                tax
                lda #<(screen1+SCROLLROWS*40-40)
                sta swdu_sta+1
                lda scradrtblhi,x
                clc
                adc #>(SCROLLROWS*40-40)
                sta swdu_sta+2
                jmp swdu_common

;Ŀ
;DRAWSCREEN                                                                   
;                                                                             
;Draws the whole gamescreen.                                                  
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y,temp1-temp6                                                  
;

drawscreen:     jsr clearsprites
                jsr initchars
                jsr sl_copytonew
                ldx newmapy
                lda maptbllo,x
                sta temp3
                lda maptblhi,x
                sta temp4
                lda newblocky
                asl
                asl
                ora newblockx
                sta temp1
                sta temp2
                ldx #$01                        ;Draw always to screen 2
                stx screen
                lda #$00
                sta swdu_sta+1
                lda scradrtblhi,x
                sta swdu_sta+2
                lda #<colors
                sta swdu_sta2+1
                lda #>colors
                sta swdu_sta2+2
                lda #SCROLLROWS
                sta temp6
drsc_loop:      jsr swdu_common2
                lda swdu_sta+1
                clc
                adc #40
                sta swdu_sta+1
                sta swdu_sta2+1
                bcc drsc_notover1
                inc swdu_sta+2
                inc swdu_sta2+2
drsc_notover1:  ldy temp1
                lda cpd_tbl,y
                bpl drsc_notover3
                pha
                lda temp3
                clc
                adc mapsizex
                sta temp3
                pla
                bcc drsc_notover3
                inc temp4
drsc_notover3:  and #$0f
                sta temp1
                sta temp2
                dec temp6
                bne drsc_loop
                jsr initscroll
                lda #$00
                sta $d01b                       ;Sprites on top of BG
                sta $d025                       ;Set sprite multicolor 1
                lda #$0a
                sta $d026                       ;Set sprite multicolor 2
                lda #SCRM_GAME                  ;Then switch gamescreen on
                sta screenmode
                rts

;Ŀ
;GETCHARPOSINFO                                                               
;                                                                             
;Gets charinfo from actor's position                                          
;                                                                             
;Parameters: X:Actor number                                                   
;Returns: A:charinfo                                                          
;Modifies: Y,mapptr                                                           
;

getcharposinfo: ldy actyh,x
                lda maptbllo,y
                sta mapptrlo
                lda maptblhi,y
                sta mapptrhi
                ldy actxh,x
                cpy mapsizex                    ;Check that we are within the
                bcs gcpi_out                    ;map
                lda (mapptrlo),y                ;Take block from map
                tay
                lda blktbllo,y
                sta mapptrlo
                lda blktblhi,y
                sta mapptrhi
                lda actyl,x
                lsr
                lsr
                lsr
                lsr
                and #12
                sta gcpi_ora+1
                lda actxl,x
                rol
                rol
                rol
                and #$03
gcpi_ora:       ora #$00
                tay
                lda (mapptrlo),y
                tay
                lda charinfo,y                  ;Take blockinfo
                rts
gcpi_out:       lda #CI_OBSTACLE                ;Checking from outside the
                rts                             ;map results in an obstacle

;Ŀ
;GETCHARPOSINFOOFFSET                                                         
;                                                                             
;Gets charinfo from actor's position, with a certain Y offset                 
;                                                                             
;Parameters: X:Actor number, A:Y offset in chars (positive or negative)       
;Returns: A:charinfo                                                          
;Modifies: Y,mapptr                                                           
;

getcharposinfooffset:
                sta gcpio_temp2+1
                lda actyl,x
                rol
                rol
                rol
                and #$03
                clc
gcpio_temp2:    adc #$00
                sta gcpio_temp+1
                bpl gcpio_pos
gcpio_neg:      lsr
                lsr
                ora #$c0
                clc
                adc actyh,x
                bpl gcpio_common
gcpio_pos:      lsr
                lsr
                clc
                adc actyh,x
gcpio_common:   tay
                lda maptbllo,y
                sta mapptrlo
                lda maptblhi,y
                sta mapptrhi
                ldy actxh,x
                cpy mapsizex
                bcs gcpi_out
                lda (mapptrlo),y                ;Take block from map
                tay
                lda blktbllo,y
                sta mapptrlo
                lda blktblhi,y
                sta mapptrhi
                lda actxl,x
                rol
                rol
                rol
                and #$03
                sta gcpio_ora+1
gcpio_temp:     lda #$00
                and #$03
                asl
                asl
gcpio_ora:      ora #$00
                tay
                lda (mapptrlo),y
                tay
                lda charinfo,y                  ;Take blockinfo
                rts

;Ŀ
;INITCHARS                                                                    
;                                                                             
;Initializes character-actors.                                                
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X                                                                
;

initchars:      ldx #MAXCHARACT-1
initcharsloop:  lda #$00                        ;Wipe flag 0: no wipe
                sta chareaddrhi,x
                dex
                bpl initcharsloop
                sta numcharactors
                rts

;Ŀ
;DRAWCHARS                                                                    
;                                                                             
;Draws all character-actors on the screen.                                    
;Will also copy the gun-sprite and do character-animation!                    
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y,temp1-temp2                                                  
;

drawchars:

;Ŀ
;DRAWSTARS                                                                    
;                                                                             
;Draws stars on empty parts of the gamescreen.                                
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y,temp1-temp2                                                  
;

drawstars:      lda colormode                   ;Stars are drawn only at
                cmp #CLMODE_MORNING             ;evening & night
                bcc drawstarsok
                jmp drawstarsdone
drawstarsok:    lda scrolly
                eor #$07
                sta starsy
                tay
                ldx scrollx
                lda pwdbittbl,x                 ;Using a table that shouldn't
                sta gamechars+8,y               ;be used, but that's exactly
                ldx screen                      ;what is needed :-)
                stx starsscreen
                lda scradrtblhi,x
                sta temp2
                lda #$00
                sta temp1
                ldx #$01
                ldy #<62
                lda (temp1),y
                bne drawstarsskip1
                txa
                sta (temp1),y
drawstarsskip1: ldy #<86
                lda (temp1),y
                bne drawstarsskip2
                txa
                sta (temp1),y
drawstarsskip2: ldy #<153
                lda (temp1),y
                bne drawstarsskip3
                txa
                sta (temp1),y
drawstarsskip3: ldy #<175
                lda (temp1),y
                bne drawstarsskip4
                txa
                sta (temp1),y
drawstarsskip4: ldy #<307
                inc temp2
                lda (temp1),y
                bne drawstarsskip5
                txa
                sta (temp1),y
drawstarsskip5: ldy #<322
                lda (temp1),y
                bne drawstarsskip6
                txa
                sta (temp1),y
drawstarsskip6: ldy #<373
                lda (temp1),y
                bne drawstarsskip7
                txa
                sta (temp1),y
drawstarsskip7: ldy #<396
                lda (temp1),y
                bne drawstarsskip8
                txa
                sta (temp1),y
drawstarsskip8: ldy #<503
                lda (temp1),y
                bne drawstarsskip9
                txa
                sta (temp1),y
drawstarsskip9: ldy #<606
                inc temp2
                lda (temp1),y
                bne drawstarsskip10
                txa
                sta (temp1),y
drawstarsskip10:ldy #<635
                lda (temp1),y
                bne drawstarsskip11
                txa
                sta (temp1),y
drawstarsskip11:ldy #<693
                lda (temp1),y
                bne drawstarsskip12
                txa
                sta (temp1),y
drawstarsskip12:
drawstarsdone:  lda numcharactors
                beq dchar_ready
                ldx #$00
                ldy screen                      ;Screen to write on
                lda scradrtblhi,y
                sta dchar_hi+1
                clc
dchar_loop:     ldy charrow,x
                lda screenrowtbllo,y
                adc charcolumn,x
                sta chareaddrlo,x
                sta temp1
                lda screenrowtblhi,y
                adc #$00
dchar_hi:       ora #$00
                sta temp2
                sta chareaddrhi,x
                ldy #$00
                lda (temp1),y                   ;Store character underneath
                sta charunder,x
                lda charnum,x
                sta (temp1),y
                sta dchar_getcolor+1
dchar_hi2:      lda temp2
                and #$03
                ora #>colors
                sta temp2
dchar_getcolor: lda charcols
                sta (temp1),y
                inx
                cpx numcharactors
                bcc dchar_loop
dchar_ready:
copygunsprite:  ldx gunframe
                bpl cgs_ok1
cgs_skip:       rts
cgs_ok1:        cpx prevgunframe
                bne cgs_ok2
                jmp cgs_onlycolor
cgs_ok2:        stx prevgunframe
N               set 0
                repeat 30
                lda weaponsprdata+N*GUNFRAMES*2,x
                sta $c000+GUNSPRITE*64+6+N
N               set N+1
                repend
cgs_onlycolor:  lda weaponsprdata+N*GUNFRAMES*2,x
                sta sprcoltbl-$40+GUNSPRITE
chra_donothing: rts

;Ŀ
;CHARANIM                                                                     
;                                                                             
;Does animation of the BG graphics.                                           
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y,temp1-temp2                                                  
;

charanim:       ldx loadedlevel
                bmi chra_donothing
                lda charanimrouthi,x
                beq chra_donothing
                sta chra_animjmp+2
                lda charanimroutlo,x
                sta chra_animjmp+1
chra_animjmp:   jmp $0000

chra_waterfall: lda gamechars+106*8+14
                pha
                ldy gamechars+106*8+15
                ldx #12
chra_waterfallloop:
                lda gamechars+106*8,x
                sta gamechars+106*8+2,x
                lda gamechars+106*8+1,x
                sta gamechars+106*8+3,x
                dex
                dex
                bpl chra_waterfallloop
                pla
                sta gamechars+106*8
                sty gamechars+106*8+1
                jsr random
                sta gamechars+114*8+5
chra_done:      rts

chra_generator: inc bganimdelay
                lda bganimdelay
                and #$03
                bne chra_done
                lda plotbits                    ;Again, this is wrong :-)
                and #$02
                pha
                asl
                asl
                ora #$07
                tay
                ldx #$07
chra_gcopy:     lda gamechars+137*8,y
                sta gamechars+136*8,x
                dey
                dex
                bpl chra_gcopy
                pla
                beq chra_blank
                lda gamechars+131*8+1
                asl
                rol gamechars+131*8+1
                asl
                rol gamechars+131*8+1
                lda gamechars+131*8+1
                sta gamechars+131*8+2
                sta gamechars+131*8+3
                sta gamechars+131*8+4
                sta gamechars+131*8+5
                sta gamechars+131*8+6
                ldy #$00
chra_gblink:    jsr random
                and #$03
                tax
                lda chra_gblinktbl,x
                sta gamechars+153*8+1,y
                sta gamechars+153*8+2,y
                iny
                iny
                iny
                iny
                cpy #$10
                bcc chra_gblink
                rts
chra_blank:     lda #%01000100
                sta gamechars+153*8+1
                sta gamechars+153*8+2
                sta gamechars+153*8+5
                sta gamechars+153*8+6
                sta gamechars+154*8+1
                sta gamechars+154*8+2
                sta gamechars+154*8+5
                sta gamechars+154*8+6
chra_done2:     rts

chra_serverroom:inc bganimdelay
                lda bganimdelay
                and #$01
                bne chra_srvroomok
                jsr random
                and #$03
                tax
                lda chra_gblinktbl,x
                sta gamechars+135*8+3
                jsr random
                and #$03
                tax
                lda chra_gblinktbl,x
                sta gamechars+139*8+2
                jsr random
                and #$03
                tax
                lda chra_gblinktbl,x
                sta gamechars+139*8+4
chra_srvroomok: rts

;Ŀ
;ERASECHARS                                                                   
;                                                                             
;Erases all character-actors from the screen.                                 
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y,temp1-temp2                                                  
;

erasechars:     ldx #MAXCHARACT-1
echar_loop:     lda chareaddrhi,x
                beq echar_skip
                sta echar_sta1+2
                and #$03
                ora #>colors
                sta echar_sta2+2
                ldy chareaddrlo,x
                lda charunder,x
echar_sta1:     sta $1000,y
                sta echar_getcolor+1
echar_getcolor: lda charcols
echar_sta2:     sta $1000,y
                lda #$00
                sta chareaddrhi,x
echar_skip:     dex
                bpl echar_loop

;Ŀ
;ERASESTARS                                                                   
;                                                                             
;Erases the stars.                                                            
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y,temp1-temp2                                                  
;

erasestars:     ldy starsy
                ldx starsscreen
                lda scradrtblhi,x
                sta temp2
                lda #$00
                sta gamechars+8,y
erasestars_ok:  sta temp1
                tax
                ldy #<62
                lda (temp1),y
                cmp #$01
                bne erasestarsskip1
                txa
                sta (temp1),y
erasestarsskip1:ldy #<86
                lda (temp1),y
                cmp #$01
                bne erasestarsskip2
                txa
                sta (temp1),y
erasestarsskip2:ldy #<153
                lda (temp1),y
                cmp #$01
                bne erasestarsskip3
                txa
                sta (temp1),y
erasestarsskip3:ldy #<175
                lda (temp1),y
                cmp #$01
                bne erasestarsskip4
                txa
                sta (temp1),y
erasestarsskip4:ldy #<307
                inc temp2
                lda (temp1),y
                cmp #$01
                bne erasestarsskip5
                txa
                sta (temp1),y
erasestarsskip5:ldy #<322
                lda (temp1),y
                cmp #$01
                bne erasestarsskip6
                txa
                sta (temp1),y
erasestarsskip6:ldy #<373
                lda (temp1),y
                cmp #$01
                bne erasestarsskip7
                txa
                sta (temp1),y
erasestarsskip7:ldy #<396
                lda (temp1),y
                cmp #$01
                bne erasestarsskip8
                txa
                sta (temp1),y
erasestarsskip8:ldy #<503
                lda (temp1),y
                cmp #$01
                bne erasestarsskip9
                txa
                sta (temp1),y
erasestarsskip9:ldy #<606
                inc temp2
                lda (temp1),y
                cmp #$01
                bne erasestarsskip10
                txa
                sta (temp1),y
erasestarsskip10:ldy #<635
                lda (temp1),y
                cmp #$01
                bne erasestarsskip11
                txa
                sta (temp1),y
erasestarsskip11:ldy #<693
                lda (temp1),y
                cmp #$01
                bne erasestarsskip12
                txa
                sta (temp1),y
erasestarsskip12:
                rts

;Ŀ
;PARALLAXEFFECT                                                               
;                                                                             
;Char rotating for the jetbike flight scene                                   
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

parallaxeffect: lda scrollcsx
                bne pe_continue
                rts
pe_continue:    lda parallaxcount1
                clc
                adc #$01
                sta parallaxcount1
                and #4+8
                asl
                ora #$07
                tay
                ldx #$07
pe_loop1:       lda gamechars+64*8,y
                sta gamechars+11*8,x
                dey
                dex
                bpl pe_loop1
                lda parallaxcount2
                clc
                adc #$02
                sta parallaxcount2
                and #4+8+16
                asl
                asl
                ora #$0f
                tay
                ldx #$0f
pe_loop2:       lda gamechars+68*8,y
                sta gamechars+12*8,x
                dey
                dex
                bpl pe_loop2
                lda parallaxcount3
                clc
                adc #$03
                sta parallaxcount3
                and #4+8+16
                asl
                asl
                ora #$0f
                tay
                ldx #$0f
pe_loop3:       lda gamechars+84*8,y
                sta gamechars+14*8,x
                dey
                dex
                bpl pe_loop3
                lda parallaxcount4
                clc
                adc #$04
                sta parallaxcount4
                and #4+8+16
                asl
                asl
                ora #$0f
                tay
                ldx #$0f
pe_loop4:       lda gamechars+100*8,y
                sta gamechars+16*8,x
                dey
                dex
                bpl pe_loop4
                rts

