;ͻ
;Metal Warrior III by Cadaver: Sprite routines                                
;Started in 5/2000!                                                           
;ͼ

;Ŀ
;INITSPRITECOLS                                                               
;                                                                             
;Initializes the spritecolors. Kernal must be switched off when called!       
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

initspritecols: ldx #$4c                        ;Start from frame $4c
                lda #<commonsprites
                sta temp1
                lda #>commonsprites
                sta temp2
                ldy #$3f
isc_loop:       sei
                dec $01
                lda (temp1),y                   ;Get color
                inc $01
                cli
                sta sprcoltbl-$40,x             ;Store to colortable
                lda temp1                       ;Go to next sprite
                clc
                adc #$40
                sta temp1
                bcc isc_nothigh
                inc temp2
isc_nothigh:    inx                             ;All sprites done?
                bne isc_loop
                rts

;Ŀ
;CLEARSPRITES                                                                 
;                                                                             
;Removes sprites from the screen.                                             
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

clearsprites:   lda #$00
                sta numsprites

;Ŀ
;SORTSPRITES                                                                  
;                                                                             
;Performs the sprite sorting & dividing into sprite-IRQs.                     
;Will wait for scorepanel-IRQ before setting the spriteupdateflag.            
;                                                                             
;Parameters: sortsprend:first unused spriteindex                              
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

sortsprites:
sspr_wait:      lda newsprflag                  ;Wait until previous "new"
                bne sspr_wait                   ;sprites have been noted by
                lda irqsprstart                 ;Use the second "half" of the
                eor #MAXSPR                     ;the spritedoublebuffer
                sta sortsprstart
                sta sortsprend
                lda irqirqstart
                eor #MAXSPRIRQ
                sta sortirqstart
                sta sortirqend
                ldy numsprites                  ;Any sprites?
                bne sspr_nz1
sspr_zerospr:   jmp sspr_alldone
sspr_nz1:       lda #$00
N               set 0
                repeat SPRROWS
                sta sprrowamount+N              ;First clear all "sort-buckets"
N               set N+1
                repend
                dey
sspr_loop1:     lda sprxh,y                     ;Check X high
                beq sspr_xok1
                cmp #$01
                bne sspr_skip1b
                lda sprxl,y                     ;Check X low
                cmp #79
                bcs sspr_skip1b
sspr_xok1:      ldx spry,y
                lda sprrowtbl,x                 ;Check Y low
                sta sprrow,y
                bmi sspr_skip1
                tax
                inc sprrowamount,x              ;Increase amount in sortbucket
sspr_skip1:     dey
                bpl sspr_loop1
                bmi sspr_loop1done
sspr_skip1b:    lda #$ff
                sta sprrow,y
                dey
                bpl sspr_loop1
sspr_loop1done: lda sortsprstart
                clc
N               set 0
                repeat SPRROWS
                sta sprrowindex+N
                adc sprrowamount+N
N               set N+1
                repend
                sta sortsprend                  ;Any passed sprites?
                cmp sortsprstart
                bne sspr_nz2
                jmp sspr_alldone
sspr_nz2:       ldy #$00
sspr_loop2:     ldx sprrow,y                    ;Now do a second loop to
                bmi sspr_skip2                  ;copy the passed sprites to
                lda sprrowindex,x               ;the sorted table
                inc sprrowindex,x
                tax
                lda spry,y
                sta sortspry,x
                lda sprxl,y
                sta sortsprxl,x
                lda sprxh,y
                beq sspr_msbok
                lda sprbittbl,x                 ;Process X-coord MSB
sspr_msbok:     sta sortsprxh,x
                lda sprf,y
                sta sortsprf,x
sspr_skip2:     iny
                cpy numsprites
                bcc sspr_loop2
                ldy sortsprstart                ;Logical spritecounter
                ldx sortirqstart                ;SpriteIRQ-counter
                txa
                adc #MAXSPRIRQ-1                ;Carry becomes 0
                sta sspr_irqlimit+1             ;Last available spriteIRQ
                tya
                sta sprirqstart,x               ;Set start of first spriteIRQ
                adc #$08
                cmp sortsprend                  ;8 or less sprites total?
                bcc sspr_notless
                ldy sortsprend
                jmp sspr_irqdone
sspr_notless:   sta sprirqend,x                 ;Set end of first spriteIRQ
                tay
                inx                             ;Onwards to next spriteIRQ
sspr_irqloop:   lda sortspry-8,y                ;Now find first suitable
                clc                             ;sprite for this spriteIRQ
                adc #20
                cmp sortspry,y
                bcc sspr_found
                iny                             ;Keep looking...
                cpy sortsprend                  ;until all logical sprites
                bcc sspr_irqloop                ;done
                bcs sspr_irqsdone
sspr_found:     tya
                sta sprirqstart,x
                adc #$08                        ;Max. 8 sprites can be loaded
                cmp sortsprend
                bcc sspr_endcmpok
                lda sortsprend
sspr_endcmpok:  sta sspr_endcmp+1               ;in the same spriteIRQ
sspr_higher:    lda sortspry,y
                sta sprirqline,x
sspr_sprloop:   iny
sspr_endcmp:    cpy #$00
                bcs sspr_irqdone
                lda sortspry-8,y                ;Can the next sprite be
                adc #13                         ;overwritten?
                cmp sprirqline,x
                bcs sspr_irqdone
                lda sortspry,y                  ;Is there yet need for it
                sbc sprirqline,x                ;to be overwritten?
                bcc sspr_higher
                cmp #20
                bcc sspr_sprloop
sspr_irqdone:   tya
                sta sprirqend,x
                inx
                cpy sortsprend
                bcs sspr_irqsdone
sspr_irqlimit:  cpx #$00                        ;Run out of spriteIRQs?
                bcc sspr_irqloop
sspr_irqsdone:  stx sortirqend
sspr_alldone:   jsr waitpanel
                inc newsprflag
                rts

