;ͻ
;Metal Warrior III by Cadaver: Weapon routines                                
;Started in 7/2000!                                                           
;ͼ

BLT_PISTOL      = 0
BLT_SHOTGUN     = 1
BLT_SMG         = 2
BLT_FLAME       = 3
BLT_MISSILE     = 4
BLT_LASER       = 5
BLT_GASFIST     = 6
BLT_SHOTGUN2    = 7
BLT_SHOTGUN3    = 8
BLT_GRENADE     = 9

BLT_FISTHIT     = 10
BLT_WHIPHIT     = 11
BLT_WHIPHIT2    = 12
BLT_KNIFEHIT    = 13
BLT_NUNCHAKUHIT = 14
BLT_MOLOTOV     = 15

DMG_BULLET      = 8
DMG_SHOTGUN     = 6
DMG_FLAME       = 4
DMG_LASER       = 8
DMG_PLASMA      = 12
DMG_MISSILE     = 16
DMG_EXPLOSION   = 16
DMG_GROUNDFLAME = 1
DMG_FIST        = 1
DMG_WHIP        = 3
DMG_KNIFE       = 4
DMG_HAWK        = 4
DMG_NUNCHAKU    = 5
DMG_MUTANTPIG   = 10
DMG_RAT         = 6
DMG_DOG         = 8
DMG_ROBOTCRUSH  = 16

MELEEDELAY      = 6
GUNLOADDELAY    = 25

;Ŀ
;ACTORSHOOT                                                                   
;                                                                             
;Shoot routine                                                                
;                                                                             
;Parameters: A:Y aiming (0=straight 1=up 2=down)                              
;            X:Actor number                                                   
;            Y:Weapon number (0=pistol 1=shotgun etc.)                        
;Returns: -                                                                   
;Modifies: A,Y,temp1-temp5                                                    
;

actorshoot:     sty temp2                       ;Weapon number
                stx temp3                       ;Actor number
                cmp #$01
                bcc as_aimdone
                beq as_up
as_down:        lda weapxspdtbl,y
                bpl as_aimdone
as_up:          lda weapxspdtbl,y
                eor #$ff
                adc #$00                        ;Carry is set at this point
as_aimdone:     sta as_yadjust+1
                lda weapdelaytbl,y
                sta as_ok+1
                ldy actt,x
                lda atyaddlo-1,y                ;Actor-dependant Y modification
                sta as_yaddlo+1
                lda atyaddhi-1,y
                sta as_yaddhi+1
                lda atxaddlo-1,y                ;Actor-dependant X modification
                sta temp4
                lda atxaddhi-1,y
                sta temp5
as_bulletloop:  ldx temp3                       ;Enemy or player bullet?
                cpx #ACTI_PLAYER
                bne as_enemy
as_player:      ldy #ACTI_FIRSTPLRBLT
                lda #ACTI_LASTPLRBLT
                bne as_common
as_enemy:       ldy #ACTI_FIRSTENEMYBLT
                lda #ACTI_LASTENEMYBLT
as_common:      jsr getfreeactor                ;Y = new actor number
                bcs as_ok
                rts
as_ok:          lda #$00
                sta actattkd,x
                lda actyl,x
                clc
as_yaddlo:      adc #$00
                sta actyl,y
                lda actyh,x
as_yaddhi:      adc #$00
                sta actyh,y
                lda actd,x                      ;Check direction
                bmi as_left
as_right:       lda actxl,x
                clc
                adc temp4
                sta actxl,y
                lda actxh,x
                adc temp5
                sta actxh,y
                ldx temp2                       ;X = weapon number
                lda weapxspdtbl,x
                sta actsx,y
                jmp as_xdone
as_left:        lda actxl,x
                sec
                sbc temp4
                sta actxl,y
                lda actxh,x
                sbc temp5
                sta actxh,y
                ldx temp2                       ;X = weapon number
                lda weapxspdtbl,x
                clc
                eor #$ff
                adc #$01
                sta actsx,y
as_xdone:       lda weapyspdtbl,x
                clc
as_yadjust:     adc #$00
                sta actsy,y
                lda weapacttbl,x
                sta actt,y
                lda weapdmgtbl,x
                sta acthp,y
                jsr bulletdirection
                and weapandtbl,x
                ora #$80                        ;Make bullet invisible for
                sta actf,y                      ;the first frame
                lda weapsndtbl,x
                beq as_skipsfx
                jsr playsfx
as_skipsfx:     lda weapnexttbl,x
                bmi as_alldone
                sta temp2
                jmp as_bulletloop
as_alldone:     ldx temp3                       ;X = actor number
                lda actt,y                      ;Is it a melee attack?
                cmp #ACT_MELEEHIT
                bne as_nomeleebonus
                lda actt,x
                tax
                lda acthp,y
                clc
                adc atdmgbonus-1,x
                sta acthp,y
                ldx temp3
as_nomeleebonus:rts

;Ŀ
;BULLETMOVE                                                                   
;                                                                             
;Highly optimal move & obstaclecheck routine for bullets                      
;                                                                             
;Parameters: X:Actor number                                                   
;Returns: A:Result of GETCHARINFO                                             
;Modifies: A,Y,temp1-temp2                                                    
;

bulletmove:     lda actsx,x
                beq bltmov_skipx
                bmi bltmov_xneg
bltmov_xpos:    clc
                adc actxl,x
                sta actxl,x
                bcc bltmov_skipx
                inc actxh,x
                jmp bltmov_skipx
bltmov_xneg:    clc
                adc actxl,x
                sta actxl,x
                bcs bltmov_skipx
                dec actxh,x
bltmov_skipx:   lda actsy,x
                beq bltmov_skipy
                bmi bltmov_yneg
bltmov_ypos:    clc
                adc actyl,x
                sta actyl,x
                bcc bltmov_skipy
                inc actyh,x
                jmp bltmov_skipy
bltmov_yneg:    clc
                adc actyl,x
                sta actyl,x
                bcs bltmov_skipy
                dec actyh,x
bltmov_skipy:   jmp getcharposinfo

;Ŀ
;BULLETDIRECTION                                                              
;                                                                             
;Finds out direction for bullet actor.                                        
;                                                                             
;Parameters: Y:Actor number                                                   
;Returns: A:Direction (0-7)                                                   
;Modifies: A,temp6-temp7                                                      
;

bulletdirection:lda actsy,y
                bmi bd_yneg
bd_ypos:        lda actsx,y
                bmi bd_yposxneg
bd_yposxpos:    lsr
                cmp actsy,y
                bcs bd_yposxposxg
                lda actsy,y
                lsr
                cmp actsx,y
                bcs bd_yposxposyg
bd_yposxpossame:lda #$03                        ;Southeast
                rts
bd_yposxposxg:  lda #$02                        ;East
                rts
bd_yposxposyg:  lda #$04                        ;South
                rts
bd_yposxneg:    clc
                eor #$ff
                adc #$01
                sta temp6
                lsr
                cmp actsy,y
                bcs bd_yposxnegxg
                lda actsy,y
                lsr
                cmp temp6
                bcs bd_yposxposyg
bd_yposxnegsame:lda #$05                        ;Southwest
                rts
bd_yposxnegxg:  lda #$06                        ;West
                rts
bd_yneg:        clc
                eor #$ff
                adc #$01
                sta temp7
                lda actsx,y
                bmi bd_ynegxneg
bd_ynegxpos:    lsr
                cmp temp7
                bcs bd_yposxposxg
                lda temp7
                lsr
                cmp actsx,y
                bcs bd_ynegxposyg
bd_ynegxpossame:lda #$01                        ;Northeast
                rts
bd_ynegxposyg:  lda #$00                        ;North
                rts
bd_ynegxneg:    clc
                eor #$ff
                adc #$01
                sta temp6
                lsr
                cmp temp7
                bcs bd_yposxnegxg
                lda temp7
                lsr
                cmp temp6
                bcs bd_ynegxposyg
bd_ynegxnegsame:lda #$07                        ;Northwest
                rts

;Ŀ
;BULLETCOLLISION                                                              
;                                                                             
;Checks bullet's collisions to enemies/player.                                
;                                                                             
;Parameters: X:Actor number                                                   
;Returns: C=0 collided (A is damage, Y is target actor), C=1 didn't collide.  
;Modifies: A,some temp variables :-)                                          
;

bulletcollision:cpx #ACTI_FIRSTPLRBLT
                bcs bltcol_player
                ldy #ACTI_PLAYER
                jsr checkpointcollision
                bcc bltcol_yes
                rts
bltcol_player:  ldy #ACTI_FIRSTOTHER
bltcol_plrloop: lda acthp,y                     ;Anything that has hitpoints,
                beq bltcol_next1                ;can be punished
                jsr checkpointcollision
                bcc bltcol_yes
bltcol_next1:   iny
                lda acthp,y
                beq bltcol_next2
                jsr checkpointcollision
                bcc bltcol_yes
bltcol_next2:   iny
                lda acthp,y
                beq bltcol_next3
                jsr checkpointcollision
                bcc bltcol_yes
bltcol_next3:   iny
                lda acthp,y
                beq bltcol_next4
                jsr checkpointcollision
                bcc bltcol_yes
bltcol_next4:   iny
                lda acthp,y
                beq bltcol_next5
                jsr checkpointcollision
                bcc bltcol_yes
bltcol_next5:   iny
                lda acthp,y
                beq bltcol_next6
                jsr checkpointcollision
                bcc bltcol_yes
bltcol_next6:   sec
                rts
bltcol_yes:     lda acthp,x
                rts

;Ŀ
;EXPLOSIONCOLLISION                                                           
;                                                                             
;Checks explosion's collision to all nearby enemies                           
;                                                                             
;Parameters: X:Actor number                                                   
;Returns: -                                                                   
;Modifies: A,Y,some temp variables :-)                                        
;

explosioncollision:
                cpx #ACTI_FIRSTPLRBLT
                bcs explcol_player
                ldy #ACTI_PLAYER
                jsr checkcollision
                bcs explcol_no
explcol_yes:    jsr faceactor
                sta actsx,x
                lda #DMG_EXPLOSION
                jmp punish
explcol_player: ldy #ACTI_FIRSTOTHER
explcol_plrloop:lda acthp,y                     ;Anything that has hitpoints
                beq explcol_next                ;can be punished
                jsr checkcollision
                bcs explcol_next
                sty temp5
                jsr explcol_yes
                ldy temp5
explcol_next:   iny
                cpy #ACTI_LASTOTHER+1
                bcc explcol_plrloop
pn_zerohp:
explcol_no:     rts

;Ŀ
;PUNISH                                                                       
;                                                                             
;Deals damage of actor number X to actor number Y                             
;                                                                             
;Parameters: A:Damage X:Punisher Y:Punished                                   
;Returns: -                                                                   
;Modifies: A,Y,temp1-temp2,temp4                                              
;

punish:         stx temp4
                pha
                asl
                asl
                asl
                sta pn_player+1
                pla
                ldx actt,y
                sec                             ;Subtract armor from damage
                sbc atarmor-1,x
                bcc pn_toolow
                bne pn_dmgok
pn_toolow:      lda #$01                        ;But damage can never become
pn_dmgok:       sta pn_sub+1                    ;less than 1
                ldx temp4
                lda acthp,y
                beq pn_zerohp
                sec
pn_sub:         sbc #$00
                bcs pn_notbelowzero
                lda #$00
pn_notbelowzero:sta acthp,y
                stx temp4
                lda actsx,x                     ;Check dir of bullet
                pha
                tya
                tax
                pla
                beq pn_noimpulse
                bmi pn_left
pn_right:       lda #16                         ;Punished receives some
                ldy #2*8                        ;impulse to the right
                bne pn_common
pn_left:        lda #-16                        ;Punished receives some
                ldy #-2*8                       ;impulse to the left
pn_common:      jsr accactorx
pn_noimpulse:   cpx #ACTI_PLAYER                ;Was player hit?
                beq pn_player
                ldy temp4
                lda actt,y                      ;Was it a melee attack?
                cmp #ACT_MELEEHIT
                bne pn_notmelee
                lda #192                        ;Give experience for each
                jsr giveexp                     ;melee hit
                jsr random
                cmp #$02                        ;Possibility of melee weapon
                bcs pn_notmelee                 ;breaking
                inc meleebreak
pn_notmelee:    lda acthp,x                     ;Did hitpoints become zero?
                bne pn_notout
                jsr enemyreward
                jmp pn_outcommon
pn_player:      lda #$00                        ;Player receives experience
                jsr giveexp                     ;for each hit taken
                lda energycheat                 ;Cheat active?
                beq pn_normalhit
                lda #MAXPLRHEALTH
                sta acthp,x
pn_normalhit:   lda acthp,x                     ;Did hitpoints become zero?
                beq pn_outcommon
pn_notout:      ldx temp4
                lda #SFX_HIT
                jmp playsfx
pn_outcommon:   ldy actt,x
                lda atout-1,y
                beq pn_outskip
                sta actt,x                      ;Then actor is out of action
                lda #$ff                        ;Reset origin (won't be placed
                sta actorg,x                    ;back to leveldata)
                lda atoutymod-1,y
                jsr moveactorydirect            ;Perform Y-modification
                lda #-14
                sta actsy,x                     ;Initial Y-speed
                lda #$00                        ;Reset frame & framedelay
                sta actf,x
                sta actfd,x
pn_outskip:     ldx temp4
                lda #SFX_EXPLOSION              ;Play explosion effect
                jmp playsfx

ar_missile:     lda actf,x
                bpl armis_notfirstframe
                and #$7f
                sta actf,x
                lda #$00
                sta actfd,x
                cpx #ACTI_FIRSTPLRBLT
                bcs armis_hometoenemy
                lda #ACTI_PLAYER
                sta actattk,x
                bpl armis_notfirstframe
armis_hometoenemy:                              ;Home to the closest enemy
                lda #$ff
                sta temp1                       ;Closest distance
                sta temp2                       ;Closest enemyindex
                ldy #ACTI_FIRSTOTHER
armis_hteloop:  lda acthp,y
                beq armis_htenext
                lda actxh,y
                sec
                sbc actxh,x
                bpl armis_htexok
                clc
                eor #$ff
                adc #$01
armis_htexok:   sta temp3
                jsr faceactor
                eor actsx,x
                and #$80
                beq armis_htenopenalty
                lda temp3                       ;Add distance "penalty"
                adc #10                         ;for targets behind the missile
                sta temp3
armis_htenopenalty:
                lda actyh,y
                sec
                sbc actyh,x
                bpl armis_hteyok
                clc
                eor #$ff
                adc #$01
armis_hteyok:   asl                             ;Y distance * 2, to make
                adc temp3                       ;targets on the same level
                cmp temp1                       ;favorable
                bcs armis_htenext
                sta temp1
                sty temp2
armis_htenext:  iny
                cpy #ACTI_LASTOTHER+1
                bcc armis_hteloop
                lda temp2
                sta actattk,x
armis_notfirstframe:
                inc actfd,x                     ;Missile explodes harmlessly
                bmi armis_explode               ;after approximately 2.5
                jsr bulletmove                  ;seconds
                and #CI_OBSTACLE
                bne armis_explode
                jsr bulletcollision
                bcs armis_home                  ;No collision
                jsr punish
armis_explode:  jmp explodeactor
armis_home:     ldy actattk,x
                bmi armis_flystraight
                cpy #ACTI_PLAYER
                beq armis_homeok
                lda acthp,y
                bne armis_homeok
armis_flystraight:
                rts
armis_homeok:   jsr faceactor
                bmi armis_turnleft
armis_turnright:lda #2
                ldy #4*8
                bne armis_xok
armis_turnleft: lda #-2
                ldy #-4*8
armis_xok:      jsr accactorx
                ldy actattk,x
                jsr faceactory
                bmi armis_turnup
armis_turndown: lda #2
                ldy #4*8
                bne armis_yok
armis_turnup:   lda #-2
                ldy #-4*8
armis_yok:      jsr accactory
                txa
                tay
                jsr bulletdirection
                sta actf,x
                rts

ar_plasma:
ar_laser:
ar_bullet:      lda actf,x                      ;Make bullet visible after
                bpl arblt_notfirstframe         ;first frame
                and #$7f
                sta actf,x
arblt_notfirstframe:
                jsr bulletmove
                and #CI_OBSTACLE
                bne arblt_remove
                jsr bulletcollision
                bcs arblt_done                  ;No collision
                jsr punish
                lda #ACT_MELEEHIT               ;If hit, show a "star"
                sta actt,x
                lda #$00
                sta actf,x
                lda #$01
                sta actfd,x
arblt_done:     rts
arblt_remove:   jmp ract_noorigin

ar_flame:       lda actf,x
                bpl arflame_notfirstframe
                and #$7f
                sta actf,x
arflame_notfirstframe:
                lda actfd,x
                clc
                adc #$60
                sta actfd,x
                bcc arblt_notfirstframe
                inc actf,x
                lda actf,x
                cmp #$05
                bcs arblt_remove
                bcc arblt_notfirstframe

ar_explosion:   lda actfd,x
                clc
                adc #$40
                sta actfd,x
                bcc ar_expldone
                inc actf,x
                lda actf,x
                cmp #$06
                bcs arblt_remove
ar_expldone:    rts

ar_groundflame: inc actfd,x
                lda actfd,x
                lsr
                lsr
                lsr
                cmp #$03
                bcs arblt_remove
                sta temp1
                jsr random
                and #$01
                clc
                adc temp1
                sta actf,x
                inc actattk,x
                lda actattk,x
                and #$01
                beq ar_expldone
                lda #$01
                jmp playerenemycoll

ar_molotov:     lda actf,x
                bpl armol_notfirstframe
                and #$7f
                sta actf,x
armol_notfirstframe:
                lda #2
                ldy #3*8
                jsr accactory
                jsr bulletmove
                ldy actsy,x
                bmi armol_obstacleonly
                and #CI_GROUND+CI_OBSTACLE
                bne armol_explode
                beq armol_noground
armol_obstacleonly:and #CI_OBSTACLE
armol_chkground:bne armol_explode
armol_noground: jsr bulletcollision
                bcc armol_explode2
                rts
armol_explode:  lda actsy,x
                and #$c0
                sta actsy,x
armol_explode2: lda #$00
                sta actsx,x
                lda #ACT_GROUNDFLAME
                ldy #SFX_SHOOT3
                jmp explodeactor2

ar_grenade:     lda actf,x
                bpl argrn_notfirstframe
                and #$7f
                sta actf,x
                jsr moveactory                  ;Move actor twice in Y-dir
                                                ;so that grenade seems to start
                                                ;higher
argrn_notfirstframe:
                lda #2
                ldy #3*8
                jsr accactory
                jsr bulletmove
                ldy actsy,x
                bmi argrn_obstacleonly
                and #CI_GROUND+CI_OBSTACLE
                bne argrn_explode
                beq argrn_noground
argrn_obstacleonly:and #CI_OBSTACLE
argrn_chkground:bne argrn_explode
argrn_noground: jsr bulletcollision
                bcc argrn_explode2
                rts
argrn_explode:  lda #-4*8
                jsr moveactorydirect
argrn_explode2: jsr explodeactor
                jmp explosioncollision
argrn_ok:       jsr bulletcollision
                bcc argrn_explode
                rts
explodeactor:   lda #ACT_EXPLOSION
                ldy #SFX_EXPLOSION
explodeactor2:  sta actt,x
                tya
                jsr playsfx
                lda #$00
                sta actf,x
                sta actfd,x
armelee_ok:     rts

ar_meleehit:    lda actfd,x
                bne armelee_notfirst
                jsr moveactorx                  ;Move "bullet" twice in X-dir
                jsr moveactorx                  ;to be sufficiently far away
                jsr moveactory
                jsr bulletcollision
                bcc armelee_collision           ;If no collision, the
                jmp arblt_remove                ;"star" is removed
armelee_collision:
                jsr punish
armelee_notfirst:
                lda actf,x                      ;Make actor visible
                and #$7f
                sta actf,x
                inc actfd,x                     ;Else show the "star" for
                lda actfd,x                     ;a while
                cmp #$06
                bcc armelee_ok
                jmp arblt_remove

